from mock import Mock, patch

from salmon import routing
from salmon.mail import MailRequest
from salmon.routing import MemoryStorage, Router, ShelveStorage, StateStorage, route

from .handlers import simple_fsm_mod
from .setup_env import SalmonTestCase, setup_router


class RoutingTestCase(SalmonTestCase):
    def test_MemoryStorage(self):
        store = MemoryStorage()
        store.set(self.__module__, "tester@localhost", "TESTED")

        self.assertEqual(store.get(self.__module__, "tester@localhost"), "TESTED")

        self.assertEqual(store.get(self.__module__, "tester2@localhost"), "START")

        store.clear()

        self.assertEqual(store.get(self.__module__, "tester@localhost"), "START")

    def test_ShelveStorage(self):
        store = ShelveStorage("run/states.db")

        store.set(self.__module__, "tester@localhost", "TESTED")
        self.assertEqual(store.get(self.__module__, "tester@localhost"), "TESTED")

        self.assertEqual(store.get(self.__module__, "tester2@localhost"), "START")

        store.clear()
        self.assertEqual(store.get(self.__module__, "tester@localhost"), "START")

    def test_RoutingBase(self):
        # check that Router is in a pristine state
        self.assertEqual(len(Router.ORDER), 0)
        self.assertEqual(len(Router.REGISTERED), 0)

        setup_router(['tests.handlers.simple_fsm_mod'])

        self.assertEqual(len(Router.ORDER), 5)
        self.assertEqual(len(Router.REGISTERED), 5)

        message = MailRequest('fakepeer', 'zedshaw@localhost', 'users-subscribe@localhost', "")
        Router.deliver(message)
        assert Router.in_state(simple_fsm_mod.CONFIRM, message)

        confirm = MailRequest('fakepeer', '"Zed Shaw" <zedshaw@localhost>',  'users-confirm-1@localhost', "")
        Router.deliver(confirm)
        assert Router.in_state(simple_fsm_mod.POSTING, message)

        Router.deliver(message)
        assert Router.in_state(simple_fsm_mod.NEXT, message)

        Router.deliver(message)
        assert Router.in_state(simple_fsm_mod.END, message)

        Router.deliver(message)
        assert Router.in_state(simple_fsm_mod.START, message)

        Router.clear_states()
        Router.LOG_EXCEPTIONS = True
        explosion = MailRequest('fakepeer', '<hacker@localhost>', 'start-explode@localhost', "")
        Router.deliver(explosion)

        assert Router.in_error(simple_fsm_mod.END, explosion)

        Router.clear_states()
        Router.LOG_EXCEPTIONS = False
        explosion = MailRequest('fakepeer',  'hacker@localhost', 'start-explode@localhost', "")
        with self.assertRaises(RuntimeError):
            Router.deliver(explosion)

        Router.reload()
        assert 'tests.handlers.simple_fsm_mod' in Router.HANDLERS
        self.assertEqual(len(Router.ORDER), 5)
        self.assertEqual(len(Router.REGISTERED), 5)

    def test_Router_undeliverable_queue(self):
        Router.clear_routes()
        Router.clear_states()

        Router.UNDELIVERABLE_QUEUE = Mock()
        msg = MailRequest('fakepeer', 'from@localhost', 'to@localhost', "Nothing")

        Router.deliver(msg)
        self.assertEqual(Router.UNDELIVERABLE_QUEUE.push.call_count, 1)

    def test_StateStorage_get_raises(self):
        s = StateStorage()
        with self.assertRaises(NotImplementedError):
            s.get("raises", "raises")

    def test_StateStorage_set_raises(self):
        s = StateStorage()
        with self.assertRaises(NotImplementedError):
            s.set("raises", "raises", "raises")

    def test_StateStorage_clear_raises(self):
        s = StateStorage()
        with self.assertRaises(NotImplementedError):
            s.clear()

    def test_route___get___raises(self):
        class BadRoute(object):

            @route("test")
            def wont_work(message, **kw):
                pass

        br = BadRoute()
        with self.assertRaises(TypeError):
            br.wont_work("raises")

    @patch('salmon.routing.reload', new=Mock(side_effect=ImportError))
    @patch('salmon.routing.LOG', new=Mock())
    def test_reload_raises(self):
        Router.LOG_EXCEPTIONS = True
        Router.reload()
        self.assertEqual(routing.LOG.exception.call_count, 1)

        Router.LOG_EXCEPTIONS = False
        routing.LOG.exception.reset_mock()
        with self.assertRaises(ImportError):
            Router.reload()
        self.assertEqual(routing.LOG.exception.call_count, 0)

        routing.LOG.exception.reset_mock()
        Router.LOG_EXCEPTIONS = True
        Router.load(['fake.handler'])
        self.assertEqual(routing.LOG.exception.call_count, 1)

        Router.LOG_EXCEPTIONS = False
        routing.LOG.exception.reset_mock()
        with self.assertRaises(ImportError):
            Router.load(['fake.handler'])
        self.assertEqual(routing.LOG.exception.call_count, 0)



from unittest import TestCase

from salmon import view
import jinja2


class ViewsTestCase(TestCase):
    def setUp(self):
        view.LOADER = jinja2.Environment(loader=jinja2.FileSystemLoader('tests/data/templates'))

    def tearDown(self):
        view.LOADER = None

    def test_load(self):
        template = view.load("template.txt")
        assert template
        assert template.render()

    def test_render(self):
        # try with some empty vars
        text = view.render({}, "template.txt")
        assert text

    def test_most_basic_form(self):
        msg = view.respond(locals(), 'template.txt')
        assert msg.Body

    def test_respond_cadillac_version(self):
        person = 'Tester'

        msg = view.respond(locals(), Body='template.txt', Html='template.html', From='test@localhost',
                           To='receiver@localhost', Subject='Test body from "%(person)s".')

        assert msg.Body
        assert msg.Html

        for k in ['From', 'To', 'Subject']:
            assert k in msg

    def test_respond_plain_text(self):
        person = 'Tester'

        msg = view.respond(locals(), Body='template.txt', From='test@localhost', To='receiver@localhost',
                           Subject='Test body from "%(person)s".')

        assert msg.Body
        assert not msg.Html

        for k in ['From', 'To', 'Subject']:
            assert k in msg

    def test_respond_html_only(self):
        person = 'Tester'

        msg = view.respond(locals(), Html='template.html', From='test@localhost', To='receiver@localhost',
                           Subject='Test body from "%(person)s".')

        assert not msg.Body
        assert msg.Html

        for k in ['From', 'To', 'Subject']:
            assert k in msg

    def test_respond_attach(self):
        person = "hello"
        mail = view.respond(locals(), Body="template.txt", From="test@localhost", To="receiver@localhost",
                            Subject='Test body from someone.')

        view.attach(mail, locals(), 'template.html', content_type="text/html", filename="template.html",
                    disposition='attachment')

        self.assertEqual(len(mail.attachments), 1)

        msg = mail.to_message()
        self.assertEqual(len(msg.get_payload()), 2)
        assert str(msg)

        mail.clear()

        view.attach(mail, locals(), 'template.html', content_type="text/html")
        self.assertEqual(len(mail.attachments), 1)

        msg = mail.to_message()
        self.assertEqual(len(msg.get_payload()), 2)
        assert str(msg)

    def test_unicode(self):
        person = u'H\xe9avy M\xe9t\xe5l Un\xeec\xf8d\xe9'
        mail = view.respond(locals(), Html="unicode.html", From="test@localhost", To="receiver@localhost",
                            Subject='Test body from someone.')

        assert str(mail)

        view.attach(mail, locals(), "unicode.html", filename="attached.html")

        assert str(mail)



import sys

from mock import Mock

from salmon import mail, utils
from salmon.routing import Router

from .setup_env import SalmonTestCase

sample_message = """From: someone@localhost
To: someone@localhost

Test
"""


def create_message():
    return mail.MailRequest("localhost", "someone@localhost", "someone@localhost", sample_message)


class HandlerTestCase(SalmonTestCase):
    def tearDown(self):
        Router.clear_routes()
        Router.clear_states()
        for key in Router.HANDLERS.keys():
            del sys.modules[key]
        Router.HANDLERS.clear()
        utils.settings = None

    def test_log_handler(self):
        import salmon.handlers.log  # noqa
        Router.deliver(create_message())

    def test_queue_handler(self):
        import salmon.handlers.queue  # noqa
        Router.deliver(create_message())

    def test_forward(self):
        utils.import_settings(False)

        import salmon.handlers.forward  # noqa
        salmon.handlers.forward.settings.relay = Mock()
        Router.deliver(create_message())

        self.assertEqual(salmon.handlers.forward.settings.relay.deliver.call_count, 1)



from __future__ import print_function

from shutil import rmtree
from unittest import TestCase
import os

from salmon.routing import Router

dirs = ("run", "logs")


def setup_router(handlers):
    Router.clear_routes()
    Router.clear_states()
    Router.HANDLERS.clear()
    Router.load(handlers)
    Router.reload()


def clean_dirs():
    for path in dirs:
        rmtree(path, ignore_errors=True)


class SalmonTestCase(TestCase):
    def setUp(self):
        clean_dirs()
        for path in dirs:
            os.mkdir(path)
        self.addCleanup(clean_dirs)



import logging
import logging.config

from salmon.routing import Router
from salmon.server import Relay

from . import settings

# configure logging to go to a log file
logging.config.fileConfig("tests/logging.conf")

# the relay host to actually send the final message to (set debug=1 to see what
# the relay is saying to the log server).
settings.relay = Relay(host=settings.relay_config['host'],
                       port=settings.relay_config['port'], debug=0)


settings.receiver = None

Router.defaults(**settings.router_defaults)
Router.load(settings.handlers + settings.queue_handlers)
Router.RELOAD = False
Router.LOG_EXCEPTIONS = False



"""
Implements a simple logging handler that's actually used by the salmon log
command line tool to run a logging server.  It simply takes every message it
receives and dumps it to the logging.debug stream.
"""

import logging

from salmon.routing import route, stateless


@route("(to)@(host)", to=".+", host=".+")
@stateless
def START(message, to=None, host=None):
    """This is stateless and handles every email no matter what, logging what it receives."""
    logging.debug("MESSAGE to %s@%s:\n%s", to, host, str(message))



# -*- coding: utf-8 -*-
from email import encoders
from email.utils import parseaddr
import mailbox
import os

from mock import Mock, patch
import chardet

from salmon import encoding

from .setup_env import SalmonTestCase

BAD_HEADERS = [
    '"\u8003\u53d6\u5206\u4eab" <Ernest.Beard@msa.hinet.net>',
    '"=?windows-1251?B?RXhxdWlzaXRlIFJlcGxpY2E=?="\n\t<wolfem@barnagreatlakes.com>',
    '=?iso-2022-jp?B?Zmlicm91c19mYXZvcmF0ZUB5YWhvby5jby5qcA==?=<fibrous_favorate@yahoo.co.jp>',

    '=?windows-1252?Q?Global_Leadership_in_HandCare_-_Consumer,\n\t_Professional_and_Industrial_Products_OTC_:_FLKI?=',
    '=?windows-1252?q?Global_Leadership_in_Handcare_-_Consumer, _Auto,\n\t_Professional_&_Industrial_Products_-_OTC_:_FLKI?=',  # noqa
    'I am just normal.',
    '=?koi8-r?B?WW91ciBtYW6ScyBzdGFtaW5hIHdpbGwgY29tZSBiYWNrIHRvIHlvdSBs?=\n\t=?koi8-r?B?aWtlIGEgYm9vbWVyYW5nLg==?=',
    '=?koi8-r?B?WW91IGNhbiBiZSBvbiB0b3AgaW4gYmVkcm9vbSBhZ2FpbiCWIGp1c3Qg?=\n\t=?koi8-r?B?YXNrIHVzIGZvciBhZHZpY2Uu?=',
    '"=?koi8-r?B?5MXMz9DSz8na18/E09TXzw==?=" <daniel@specelec.com>',
    '=?utf-8?b?IumrlOiCsuWckuWNgOermSDihpIg6ZW35bqa6Yar6Zmi56uZIOKGkiDmlofljJbk?=\n =?utf-8?b?uInot6/nq5kiIDx2Z3hkcmp5Y2lAZG5zLmh0Lm5ldC50dz4=?=',  # noqa: E501
    '=?iso-8859-1?B?SOlhdnkgTel05WwgVW7uY/hk?=\n\t=?iso-8859-1?Q?=E9?=',
]

DECODED_HEADERS = encoding.header_from_mime_encoding(BAD_HEADERS)

NORMALIZED_HEADERS = [encoding.header_to_mime_encoding(x) for x in DECODED_HEADERS]


class EncodingTestCase(SalmonTestCase):
    def test_MailBase(self):
        the_subject = u'p\xf6stal'
        m = encoding.MailBase()

        m['To'] = "testing@localhost"
        m['Subject'] = the_subject
        m['Content-Type'] = 'text/plain; charset=iso-8859-1'
        m['MIME-Version'] = '1.0'

        self.assertEqual(m['To'], "testing@localhost")
        self.assertEqual(m['TO'], m['To'])
        self.assertEqual(m['to'], m['To'])

        self.assertEqual(m['Subject'], the_subject)
        self.assertEqual(m['subject'], m['Subject'])
        self.assertEqual(m['sUbjeCt'], m['Subject'])

        msg = encoding.to_message(m)
        m2 = encoding.from_message(msg)

        self.assertEqual(len(m), len(m2))

        self.assertEqual(m.items(), m2.items())

        for k in m:
            assert m[k] == m2[k], "%s: %r != %r" % (k, m[k], m2[k])

        assert("To" in m)
        assert("Bob" not in m)
        self.assertEqual(m["To"], "testing@localhost")
        m["To"] = "somebody@localhost"
        self.assertEqual(m["To"], "somebody@localhost")
        self.assertEqual(m.keys(), ['Subject', 'Mime-Version', 'Content-Type', 'To'])
        self.assertEqual(m.items(), [
            ('Subject', the_subject),
            ('Mime-Version', '1.0'),
            ('Content-Type', 'text/plain; charset="iso-8859-1"'),
            ('To', 'somebody@localhost')])
        self.assertEqual(m.get_all("Subject"), [the_subject])
        self.assertEqual(m.get_all("Bob"), [])

        # appending headers
        m.append_header("subject", "something else")
        self.assertEqual(m["Subject"], the_subject)
        self.assertEqual(m.keys(), ['Subject', 'Mime-Version', 'Content-Type', 'To', "Subject"])
        self.assertEqual(m.items(), [
            ('Subject', the_subject),
            ('Mime-Version', '1.0'),
            ('Content-Type', 'text/plain; charset="iso-8859-1"'),
            ('To', 'somebody@localhost'),
            ('Subject', "something else")])
        self.assertEqual(m.get_all("Subject"), [the_subject, "something else"])

        # reset subject header
        m["Subject"] = the_subject

        for key, item in zip(m.keys(), m.items()):
            self.assertEqual(key, item[0])
            assert key in m
            del m[key]
            assert key not in m
            assert key not in [i[0] for i in m.items()]

    def test_normalized_headers(self):
        base = encoding.MailBase()

        # __setitem__ will normalize headers when they're added
        base["subJect"] = "test"
        self.assertEqual(base["Subject"], "test")
        self.assertEqual(base.mime_part["Subject"], "test")
        self.assertEqual(base.keys(), ["Subject"])
        self.assertEqual(base.mime_part.keys(), ["Subject"])

        # if the header wasn't added via __setitem__, it should still be returned
        # to Salmon users in a normalized form
        base.mime_part["fRom"] = "test@example.com"
        self.assertEqual(base["From"], "test@example.com")
        self.assertEqual(base.mime_part["fRom"], "test@example.com")
        self.assertEqual(base.keys(), ["Subject", "From"])
        self.assertEqual(base.mime_part.keys(), ["Subject", "fRom"])

    def test_header_to_mime_encoding(self):
        for i, header in enumerate(DECODED_HEADERS):
            self.assertEqual(NORMALIZED_HEADERS[i], encoding.header_to_mime_encoding(header))

    def test_mutt_badness(self):
        # this is a sample of possibly the worst case Mutt can produce
        idiot = '=?iso-8859-1?B?SOlhdnkgTel05WwgVW7uY/hk?=\n\t=?iso-8859-1?Q?=E9?='
        should_be = u'H\xe9avy M\xe9t\xe5l Un\xeec\xf8d\xe9'
        self.assertEqual(encoding.header_from_mime_encoding(idiot), should_be)

    def test_header_from_mime_encoding(self):
        assert not encoding.header_from_mime_encoding(None)
        self.assertEqual(len(BAD_HEADERS), len(encoding.header_from_mime_encoding(BAD_HEADERS)))

        for i, header in enumerate(BAD_HEADERS):
            self.assertEqual(DECODED_HEADERS[i], encoding.header_from_mime_encoding(header))

    def test_to_message_from_message_with_spam(self):
        mb = mailbox.mbox("tests/data/spam")

        for msg in mb:
            m = encoding.from_message(msg)
            out = encoding.to_message(m)
            assert repr(out)

            m2 = encoding.from_message(out)

            for k in m:
                if '@' in m[k]:
                    self.assertEqual(parseaddr(m[k]), parseaddr(m2[k]))
                elif k.lower() in [key.lower() for key in encoding.CONTENT_ENCODING_KEYS]:
                    pass  # skip!
                else:
                    assert m[k].strip() == m2[k].strip(), "%s: %r != %r" % (k, m[k], m2[k])

                assert not m[k].startswith(u"=?")
                assert not m2[k].startswith(u"=?")

            # salmon adds stuff to content headers that's missing from example messages
            for k in encoding.CONTENT_ENCODING_KEYS:
                if k == "Content-Transfer-Encoding" \
                        and m.content_encoding[k][0] is None \
                        and m2.content_encoding[k][0] == "7bit":
                    continue  # salmon fixing bad email
                elif k == "Mime-Version" \
                        and m.content_encoding[k][0] is None \
                        and m2.content_encoding[k][0] == "1.0":
                    continue  # again, salmon fixing bad email

                assert m.content_encoding[k][0] == m2.content_encoding[k][0], \
                    "%s: %r != %r" % (k, m.content_encoding[k], m2.content_encoding[k])

                for p in m.content_encoding[k][1]:
                    if p in m2.content_encoding[k][1]:
                        assert m.content_encoding[k][1][p].lower() == m2.content_encoding[k][1][p].lower(), \
                            "%s: %r != %r" % (p, m.content_encoding[k][1][p], m2.content_encoding[k][1][p])

            assert m.body == m2.body, "Bodies don't match"

            self.assertEqual(len(m.parts), len(m2.parts), "Not the same number of parts.")

            for i, part in enumerate(m.parts):
                assert part.body == m2.parts[i].body, \
                    "Part %d isn't the same: %r \nvs\n. %r" % (i, part.body, m2.parts[i].body)

    def test_to_file_from_file(self):
        mb = mailbox.mbox("tests/data/spam")
        msg = encoding.from_message(mb[0])

        outfile = "run/encoding_test.msg"

        with open(outfile, 'w') as outfp:
            encoding.to_file(msg, outfp)

        with open(outfile) as outfp:
            msg2 = encoding.from_file(outfp)

        self.assertEqual(len(msg), len(msg2))
        os.unlink(outfile)

    def test_guess_encoding_and_decode(self):
        for header in DECODED_HEADERS:
            try:
                encoding.guess_encoding_and_decode('ascii', header.encode('utf-8'))
            except encoding.EncodingError:
                pass

    def test_attempt_decoding(self):
        for header in DECODED_HEADERS:
            encoding.attempt_decoding('ascii', header.encode('utf-8'))

    def test_properly_decode_header(self):
        for i, header in enumerate(BAD_HEADERS):
            parsed = encoding.properly_decode_header(header)
            self.assertEqual(DECODED_HEADERS[i], parsed)

    def test_headers_round_trip(self):
        # round trip the headers to make sure they convert reliably back and forth
        for header in BAD_HEADERS:
            original = encoding.header_from_mime_encoding(header)

            assert original
            assert "=?" not in original and "?=" not in original, "Didn't decode: %r" % header

            encoded = encoding.header_to_mime_encoding(original)
            assert encoded

            return_original = encoding.header_from_mime_encoding(encoded)
            self.assertEqual(original, return_original)

            return_encoded = encoding.header_to_mime_encoding(return_original)
            self.assertEqual(encoded, return_encoded)

    def test_MIMEPart(self):
        text1 = encoding.MIMEPart("text/plain")
        text1.set_payload("The first payload.")
        text2 = encoding.MIMEPart("text/plain")
        text2.set_payload("The second payload.")

        with open("tests/data/salmon.png", "rb") as file_obj:
            image_data = file_obj.read()
        img1 = encoding.MIMEPart("image/png")
        img1.set_payload(image_data)
        img1.set_param('attachment', '', header='Content-Disposition')
        img1.set_param('filename', 'salmon.png', header='Content-Disposition')
        encoders.encode_base64(img1)

        multi = encoding.MIMEPart("multipart/mixed")
        for x in [text1, text2, img1]:
            multi.attach(x)

        mail = encoding.from_message(multi)

        self.assertEqual(mail.parts[0].body, "The first payload.")
        self.assertEqual(mail.parts[1].body, "The second payload.")
        self.assertEqual(mail.parts[2].body, image_data)

        encoding.to_message(mail)

    @patch('chardet.detect', new=Mock())
    def test_guess_encoding_fails_completely(self):
        chardet.detect.return_value = {'encoding': None, 'confidence': 0.0}
        with self.assertRaises(encoding.EncodingError):
            encoding.guess_encoding_and_decode('ascii', 'some data', errors='strict')

    def test_attach_text(self):
        mail = encoding.MailBase()
        mail.attach_text("This is some text.", 'text/plain')

        msg = encoding.to_message(mail)
        self.assertEqual(msg.get_payload(0).get_payload(), "This is some text.")
        assert encoding.to_string(mail)

        mail.attach_text("<html><body><p>Hi there.</p></body></html>", "text/html")
        msg = encoding.to_message(mail)
        self.assertEqual(len(msg.get_payload()), 2)
        assert encoding.to_string(mail)

    def test_attach_file(self):
        mail = encoding.MailBase()
        with open("tests/data/salmon.png", "rb") as file_obj:
            png = file_obj.read()
        mail.attach_file("salmon.png", png, "image/png", "attachment")
        msg = encoding.to_message(mail)

        payload = msg.get_payload(0)
        self.assertEqual(payload.get_payload(decode=True), png)
        self.assertEqual(payload.get_filename(), "salmon.png")

    def test_content_encoding_headers_are_maintained(self):
        with open("tests/data/signed.msg", "rb") as file_obj:
            inmail = encoding.from_file(file_obj)

        ctype, ctype_params = inmail.content_encoding['Content-Type']

        self.assertEqual(ctype, 'multipart/signed')

        # these have to be maintained
        for key in ['protocol', 'micalg']:
            assert key in ctype_params

        # these get removed
        for key in encoding.CONTENT_ENCODING_REMOVED_PARAMS:
            assert key not in ctype_params

        outmsg = encoding.to_message(inmail)
        ctype, ctype_params = encoding.parse_parameter_header(outmsg, 'Content-Type')
        for key in ['protocol', 'micalg']:
            assert key in ctype_params, key

    def test_odd_content_type_with_charset(self):
        mail = encoding.MailBase()
        mail.body = u"p\xf6stal".encode('utf-8')
        mail.content_encoding['Content-Type'] = ('application/plain', {'charset': 'utf-8'})

        msg = encoding.to_string(mail)
        assert msg

    def test_specially_borked_lua_message(self):
        with open("tests/data/borked.msg", "rb") as file_obj:
            msg = encoding.from_file(file_obj)
        assert msg

    @patch('salmon.encoding.MIMEPart.__init__')
    def test_to_message_encoding_error(self, mp_init):
        mp_init.side_effect = TypeError
        with open("tests/data/borked.msg", "rb") as file_obj:
            test = encoding.from_file(file_obj)
        with self.assertRaises(encoding.EncodingError):
            encoding.to_message(test)

    @patch('salmon.encoding.chardet.detect')
    def test_guess_encoding_and_decode_unicode_error(self, detect_mock):
        data = u"testß".encode("latin-1")
        detect_mock.return_value = {"confidence": 0.99, "encoding": "utf-8"}
        with self.assertRaises(encoding.EncodingError):
            encoding.guess_encoding_and_decode("ascii", data)

    def test_attempt_decoding_with_bad_encoding_name(self):
        self.assertEqual("test", encoding.attempt_decoding("asdfasdf", "test"))

    def test_apply_charset_to_header_with_bad_encoding_char(self):
        with self.assertRaises(encoding.EncodingError):
            encoding.apply_charset_to_header('ascii', 'X', 'bad')

    def test_odd_roundtrip_bug(self):
        decoded_addrs = [
                u'"\u0414\u0435\u043b\u043e\u043f\u0440\u043e\u0438\u0437\u0432\u043e\u0434\u0441\u0442\u0432\u043e" <daniel@specelec.com>',  # noqa: E501
            u'"\u8003\u53d6\u5206\u4eab" <Ernest.Beard@msa.hinet.net>',
            u'"Exquisite Replica"\n\t<wolfem@barnagreatlakes.com>',
        ]

        for decoded in decoded_addrs:
            encoded = encoding.header_to_mime_encoding(decoded)
            assert '<' in encoded and '"' in encoded, "Address wasn't encoded correctly:\n%s" % encoded

    def test_multiple_headers(self):
        msg = encoding.MailBase()
        msg["To"] = "somedude@localhost"
        msg["From"] = "nobody@localhost"
        assert("From" in msg)
        self.assertEqual(msg["From"], "nobody@localhost")
        msg["From"] = "somebody@localhost"
        self.assertEqual(msg["From"], "somebody@localhost")
        self.assertEqual(msg.keys(), ["To", "From"])

        # appending headers
        msg.append_header("To", "nobody@example.com")
        self.assertEqual(msg["To"], "somedude@localhost")
        self.assertEqual(msg.keys(), ["To", "From", "To"])



from salmon import mail
from salmon.routing import Router

from .handlers import bounce_filtered_mod
from .setup_env import SalmonTestCase


class BounceTestCase(SalmonTestCase):
    def setUp(self):
        super(BounceTestCase, self).setUp()
        Router.load(["tests.handlers.bounce_filtered_mod"])
        Router.reload()

    def tearDown(self):
        super(BounceTestCase, self).tearDown()
        Router.HANDLERS.clear()
        Router.reload()

    def test_bounce_analyzer_on_bounce(self):
        with open("tests/data/bounce.msg") as file_obj:
            bm = mail.MailRequest(None, None, None, file_obj.read())
        assert bm.is_bounce()
        assert bm.bounce
        self.assertEqual(bm.bounce.score, 1.0)
        assert bm.bounce.probable()
        self.assertEqual(bm.bounce.primary_status, (5, u'Permanent Failure'))
        self.assertEqual(bm.bounce.secondary_status, (1, u'Addressing Status'))
        self.assertEqual(bm.bounce.combined_status, (11, u'Bad destination mailbox address'))

        assert bm.bounce.is_hard()
        self.assertEqual(bm.bounce.is_hard(), not bm.bounce.is_soft())

        self.assertEqual(bm.bounce.remote_mta, u'gmail-smtp-in.l.google.com')
        self.assertEqual(bm.bounce.reporting_mta, u'mail.zedshaw.com')
        self.assertEqual(bm.bounce.final_recipient,
                         u'asdfasdfasdfasdfasdfasdfewrqertrtyrthsfgdfgadfqeadvxzvz@gmail.com')
        self.assertEqual(bm.bounce.diagnostic_codes[0], u'550-5.1.1')
        self.assertEqual(bm.bounce.action, 'failed')
        assert 'Content-Description-Parts' in bm.bounce.headers

        assert bm.bounce.error_for_humans()

    def test_bounce_analyzer_on_regular(self):
        with open("tests/data/signed.msg") as file_obj:
            bm = mail.MailRequest(None, None, None, file_obj.read())
        assert not bm.is_bounce()
        assert bm.bounce
        self.assertEqual(bm.bounce.score, 0.0)
        assert not bm.bounce.probable()
        self.assertEqual(bm.bounce.primary_status, (None, None))
        self.assertEqual(bm.bounce.secondary_status, (None, None))
        self.assertEqual(bm.bounce.combined_status, (None, None))

        assert not bm.bounce.is_hard()
        assert not bm.bounce.is_soft()

        self.assertEqual(bm.bounce.remote_mta, None)
        self.assertEqual(bm.bounce.reporting_mta, None)
        self.assertEqual(bm.bounce.final_recipient, None)
        self.assertEqual(bm.bounce.diagnostic_codes, [None, None])
        self.assertEqual(bm.bounce.action, None)

    def test_bounce_to_decorator(self):
        with open("tests/data/bounce.msg") as file_obj:
            msg = mail.MailRequest(None, None, None, file_obj.read())

        Router.deliver(msg)
        assert Router.in_state(bounce_filtered_mod.START, msg)
        assert bounce_filtered_mod.HARD_RAN, "Hard bounce state didn't actually run: %r" % msg.To

        msg.bounce.primary_status = (4, u'Persistent Transient Failure')
        Router.clear_states()
        Router.deliver(msg)
        assert Router.in_state(bounce_filtered_mod.START, msg)
        assert bounce_filtered_mod.SOFT_RAN, "Soft bounce didn't actually run."

        with open("tests/data/signed.msg") as file_obj:
            msg = mail.MailRequest(None, None, None, file_obj.read())
        Router.clear_states()
        Router.deliver(msg)
        assert Router.in_state(bounce_filtered_mod.END, msg), "Regular messages aren't delivering."

    def test_bounce_getting_original(self):
        with open("tests/data/bounce.msg") as file_obj:
            msg = mail.MailRequest(None, None, None, file_obj.read())
        msg.is_bounce()

        assert msg.bounce.notification
        assert msg.bounce.notification.body

        assert msg.bounce.report

        for part in msg.bounce.report:
            assert [(k, part[k]) for k in part]
            # these are usually empty, but might not be.  they are in our test
            assert not part.body

        assert msg.bounce.original
        self.assertEqual(msg.bounce.original['to'], msg.bounce.final_recipient)
        assert msg.bounce.original.body

    def test_bounce_no_headers_error_message(self):
        msg = mail.MailRequest(None, None, None, "Nothing")
        msg.is_bounce()
        self.assertEqual(msg.bounce.error_for_humans(), 'No status codes found in bounce message.')



"""
Documentation for this module can be found in :doc:`commandline`
"""

from __future__ import unicode_literals

from importlib import import_module
import glob
import mailbox
import os
import shutil
import signal
import socket
import sys

import click

from salmon import encoding, mail
from salmon import queue as queue_module
from salmon import routing, server, utils
import salmon

# squash warning about unicode literals. if there are bugs here, then it's
# quite likely to have afffected us before switching to click.
click.disable_unicode_literals_warning = True

DEFAULT_PID_FILE = "./run/smtp.pid"

copyright_notice = """
Salmon is Copyright (C) Matt Molyneaux 2014-2015, 2019.  Licensed GPLv3.
Forked from Lamon, Copyright (C) Zed A. Shaw 2008-2009.  Licensed GPLv3.
If you didn't get a copy of the LICENSE go to:

    https://github.com/moggers87/salmon/LICENSE

Have fun.
"""

uid_desc = """
If you specify a uid/gid then this means you want to first change to
root, set everything up, and then drop to that UID/GID combination.
This is typically so you can bind to port 25 and then become "safe"
to continue operating as a non-root user. If you give one or the other,
this it will just change to that uid or gid without doing the priv drop
operation.
"""


class SalmonCommandError(click.ClickException):
    """Like ClickException, but it doesn't prepend messages with "Error"

    Useful if you want to give a non-zero exit code that's not really an error
    per se
    """
    def show(self, file=None):
        click.echo(self.format_message(), err=True, file=file)


def daemon_start(command, additional_options=[], *args, **kwargs):
    """Wrap a command function with common daemon start parameters"""
    def inner(fn):
        options = additional_options + [
            click.option("--chroot", metavar="PATH", help="path to chroot"),
            click.option("--chdir", metavar="PATH", default=".", help="change to this directory when daemonising"),
            click.option("--umask", metavar="MASK", type=int, help="set umask on server"),
            click.option("--pid", metavar="PATH", default=DEFAULT_PID_FILE, help="path to pid file"),
            click.option("-f", "--force", default=False, is_flag=True, help="force server to run, ignoring pid file"),
            click.option("--debug", default=False, is_flag=True, help="debug mode"),
            click.option("--uid", metavar="UID", type=int, help="run with this user id"),
            click.option("--gid", metavar="GID", type=int, help="run with this group id"),
            click.option("--daemon/--no-daemon", default=True, help="start server as daemon (default)"),
        ]

        for option in reversed(options):
            fn = option(fn)
        kwargs.setdefault("epilog", uid_desc)
        fn = command(*args, **kwargs)(fn)

        return fn
    return inner


@click.group(epilog=copyright_notice)
@click.version_option()
def main():
    """Python mail server"""
    pass


@daemon_start(main.command, additional_options=[
    click.option("--port", metavar="PORT", default=8825, type=int, help="port to listen on"),
    click.option("--host", metavar="ADDRESS", default="127.0.0.1", help="address to listen on"),
], short_help="starts log server")
def log(port, host, pid, chdir, chroot, uid, gid, umask, force, debug, daemon):
    """
    Runs a logging only server on the given hosts and port.  It logs
    each message it receives and also stores it to the run/queue
    so that you can make sure it was received in testing.
    """
    utils.start_server(pid, force, chroot, chdir, uid, gid, umask,
                       lambda: utils.make_fake_settings(host, port), debug, daemon)


@main.command(short_help="send a new email")
@click.option("--port", metavar="PORT", default=8825, type=int, help="Port to connect to")
@click.option("--host", metavar="ADDRESS", default="127.0.0.1", help="Host to connect to")
@click.option("--username", help="SMTP username")
@click.option("--password", help="SMTP password")
@click.option("--sender", metavar="EMAIL")
@click.option("--to", metavar="EMAIL")
@click.option("--subject")
@click.option("--body")
@click.option("--attach")
@click.option("--lmtp", default=False)
@click.option("--ssl", default=False)
@click.option("--starttls", default=False)
def send(port, host, username, password, ssl, starttls, lmtp, sender, to,
         subject, body, attach):
    """
    Sends an email to someone as a test message.
    See the sendmail command for a sendmail replacement.
    """
    message = mail.MailResponse(From=sender, To=to, Subject=subject, Body=body)
    if attach:
        message.attach(attach)

    relay = server.Relay(host, port=port, username=username, password=password, ssl=ssl,
                         starttls=starttls, lmtp=lmtp, debug=False)
    relay.deliver(message)


@main.command(short_help="send an email from stdin")
@click.option("--port", metavar="PORT", default=8825, type=int, help="Port to connect to")
@click.option("--host", metavar="ADDRESS", default="127.0.0.1", help="Address to connect to")
@click.option("--lmtp", default=False, is_flag=True, help="Use LMTP rather than SMTP")
@click.option("--debug", default=False, is_flag=True, help="Debug mode")
@click.argument("recipients", nargs=-1, required=True)
def sendmail(port, host, recipients, debug, lmtp):
    """
    Used as a testing sendmail replacement for use in programs
    like mutt as an MTA.  It reads the email to send on the stdin
    and then delivers it based on the port and host settings.
    """
    relay = server.Relay(host, port=port, debug=debug, lmtp=lmtp)
    data = sys.stdin.read()
    msg = mail.MailRequest(None, recipients, None, data)
    relay.deliver(msg)


@daemon_start(main.command, additional_options=[
    click.option("--boot", metavar="MODULE", default="config.boot", help="module with server definition"),
], short_help="starts a server")
def start(pid, force, chdir, boot, chroot, uid, gid, umask, debug, daemon):
    """
    Runs a salmon server out of the current directory
    """
    utils.start_server(pid, force, chroot, chdir, uid, gid, umask,
                       lambda: utils.import_settings(True, boot_module=boot), debug, daemon)


@main.command(short_help="stops a server")
@click.option("--pid", metavar="PATH", default=DEFAULT_PID_FILE, help="path to pid file")
@click.option("-f", "--force", default=False, is_flag=True, help="force stop server")
@click.option("--all", "all_pids", help="stops all servers with .pid files in the specified directory")
def stop(pid, force, all_pids):
    """
    Stops a running salmon server
    """
    pid_files = []

    if all_pids:
        pid_files = glob.glob(all_pids + "/*.pid")
    else:
        pid_files = [pid]

        if not os.path.exists(pid):
            raise click.FileError(pid, "Maybe Salmon isn't running?")

    click.echo("Stopping processes with the following PID files: %s" % pid_files)

    for pid_f in pid_files:
        with open(pid_f) as pid_file:
            pid_data = pid_file.readline()
            click.echo("Attempting to stop salmon at pid %d" % int(pid_data))

        try:
            if force:
                os.kill(int(pid_data), signal.SIGKILL)
            else:
                os.kill(int(pid_data), signal.SIGHUP)

            os.unlink(pid_f)
        except OSError as exc:
            raise click.ClickException("stopping Salmon on PID %d: %s" % (int(pid_data), exc))


@main.command(short_help="displays status of server")
@click.option("--pid", metavar="PATH", default=DEFAULT_PID_FILE, help="path to pid file")
def status(pid):
    """
    Prints out status information about salmon useful for finding out if it's
    running and where.
    """
    if os.path.exists(pid):
        with open(pid) as pid_file:
            pid_data = pid_file.readline()
            click.echo("Salmon running with PID %d" % int(pid_data))
    else:
        raise SalmonCommandError("Salmon not running.")


@main.command(short_help="manipulate a Queue")
@click.option("--pop", default=False, is_flag=True, help="pop a message from queue")
@click.option("--get", metavar="KEY", help="get key from queue")
@click.option("--remove", metavar="KEY", help="remove chosen key from queue")
@click.option("--count", default=False, is_flag=True, help="count messages in queue")
@click.option("--clear", default=False, is_flag=True, help="clear queue")
@click.option("--keys", default=False, is_flag=True, help="print queue keys")
@click.argument("name", metavar="PATH", default="./run/queue")
def queue(name, pop, get, keys, remove, count, clear):
    """
    Lets you do most of the operations available to a queue.
    """
    click.echo("Using queue: %r" % name)

    inq = queue_module.Queue(name)

    if pop:
        key, msg = inq.pop()
        if key:
            click.echo("KEY: %s" % key)
            click.echo(msg)
    elif get:
        click.echo(inq.get(get))
    elif remove:
        inq.remove(remove)
    elif count:
        click.echo("Queue %s contains %d messages" % (name, len(inq)))
    elif clear:
        inq.clear()
    elif keys:
        click.echo("\n".join(inq.keys()))


@main.command(short_help="display routes")
@click.option("--path", metavar="PATH", default=os.getcwd, help="search path for modules")
@click.option("--test", metavar="EMAIL", help="address to test against routing configuration")
@click.argument("modules", metavar="MODULE", nargs=-1, required=True)
def routes(modules, test, path):
    """
    Prints out valuable information about an application's routing configuration
    after everything is loaded and ready to go.  Helps debug problems with
    messages not getting to your handlers.  Path has the search paths you want
    separated by a ':' character, and it's added to the sys.path.

    MODULE should be a configureation module and can be given multiple times.
    """
    sys.path += path.split(':')
    test_case_matches = []

    utils.import_settings(False)

    for module in modules:
        try:
            import_module(module)
        except ImportError:
            raise click.ClickException("Module '%s' could not be imported. Did you forget to use the --path option?"
                                       % str(module))

    if not routing.Router.REGISTERED:
        raise click.ClickException("Modules '%s' imported, but no function registered." % str(modules))

    # TODO: stop casting everything to str once Python 2.7 support has been dropped
    # we do this to avoid spamming u"blah blah" everywhere
    click.echo("Routing ORDER: %s" % [str(i) for i in routing.Router.ORDER])
    click.echo("Routing TABLE:\n---")
    for format in routing.Router.REGISTERED:
        click.echo("%r: " % str(format), nl=False)
        regex, functions = routing.Router.REGISTERED[format]
        for func in functions:
            click.echo("%s.%s " % (func.__module__, func.__name__), nl=False)
            if test:
                match = regex.match(test)
                if match:
                    test_case_matches.append((str(format), func, match))

        click.echo("\n---")

    if test_case_matches:
        click.echo("\nTEST address %r matches:" % str(test))
        for format, func, match in test_case_matches:
            click.echo("  %r %s.%s" % (format, func.__module__, func.__name__))
            click.echo("  -  %r" % ({str(k): str(v) for k, v in match.groupdict().items()}))
    elif test:
        click.echo("\nTEST address %r didn't match anything." % str(test))
        # don't raise a ClickException because that prepends "ERROR" to the
        # output and this isn't always an error
        sys.exit(1)


@main.command(short_help="generate a new project")
@click.argument("project", metavar="PATH")
@click.option("-f", "--force", is_flag=True, help="overwrite existing directories")
def gen(project, force):
    """
    Generates various useful things for you to get you started.
    """
    template = os.path.join(salmon.__path__[0], "data", "prototype")

    if force:
        shutil.rmtree(project, ignore_errors=True)
    elif os.path.exists(project):
        raise click.ClickException("Project '%s' exists, delete it first." % project)

    shutil.copytree(template, project)


@main.command(short_help="cleanse your emails")
@click.argument("inbox", metavar="IN_MAILBOX")
@click.argument("outbox", metavar="OUT_MAILBOX")
def cleanse(inbox, outbox):
    """
    Uses Salmon mail cleansing and canonicalization system to take an
    input Maildir (or mbox) and replicate the email over into another
    Maildir.  It's used mostly for testing and cleaning.
    """
    error_count = 0

    try:
        inbox = mailbox.mbox(inbox, create=False)
    except (mailbox.Error, IOError):
        try:
            inbox = mailbox.Maildir(inbox, factory=None, create=False)
        except (mailbox.Error, IOError):
            raise click.ClickException("{} does not exist or is not a valid MBox or Maildir".format(inbox))
    outbox = mailbox.Maildir(outbox)

    for msg in inbox:
        try:
            mail = encoding.from_message(msg)
            outbox.add(encoding.to_string(mail))
        except encoding.EncodingError as exc:
            click.echo("ERROR: %s" % exc)
            error_count += 1

    outbox.close()
    inbox.close()

    if error_count > 0:
        raise SalmonCommandError("TOTAL ERRORS: %s" % error_count)
    else:
        click.echo("Completed without errors")


@main.command(short_help="blast emails at a server")
@click.argument("inbox", metavar="MAILBOX")
@click.option("--port", metavar="PORT", default=8823, type=int, help="port to connect to")
@click.option("--host", metavar="ADDRESS", default="127.0.0.1", help="address to connect to")
@click.option("--lmtp", default=False, is_flag=True)
@click.option("--debug", default=False, is_flag=True, help="debug mode")
def blast(inbox, host, port, lmtp, debug):
    """
    Given a Maildir, this command will go through each email
    and blast it at your server.  It does nothing to the message, so
    it will be real messages hitting your server, not cleansed ones.
    """
    try:
        inbox = mailbox.mbox(inbox, create=False)
    except (mailbox.Error, IOError):
        try:
            inbox = mailbox.Maildir(inbox, factory=None, create=False)
        except (mailbox.Error, IOError):
            raise click.ClickException("{} does not exist or is not a valid MBox or Maildir".format(inbox))

    relay = server.Relay(host, port=port, lmtp=lmtp, debug=debug)

    for key in inbox.keys():
        msgfile = inbox.get_file(key)
        msg = encoding.from_file(msgfile)
        try:
            relay.deliver(msg)
        except socket.error as exp:
            raise click.ClickException(str(exp))
        finally:
            msgfile.close()



"""
Simpler queue management than the regular mailbox.Maildir stuff.  You
do get a lot more features from the Python library, so if you need
to do some serious surgery go use that.  This works as a good
API for the 90% case of "put mail in, get mail out" queues.
"""
from __future__ import print_function, unicode_literals

import errno
import hashlib
import logging
import mailbox
import os
import socket
import time

import six

from salmon import mail

# we calculate this once, since the hostname shouldn't change for every
# email we put in a queue
HASHED_HOSTNAME = hashlib.md5(socket.gethostname().encode("utf-8")).hexdigest()


class SafeMaildir(mailbox.Maildir):
    def _create_tmp(self):
        now = time.time()
        uniq = "%s.M%sP%sQ%s.%s" % (int(now), int(now % 1 * 1e6), os.getpid(),
                                    mailbox.Maildir._count, HASHED_HOSTNAME)
        path = os.path.join(self._path, 'tmp', uniq)
        try:
            os.stat(path)
        except OSError as e:
            if e.errno == errno.ENOENT:
                mailbox.Maildir._count += 1
                try:
                    return mailbox._create_carefully(path)
                except OSError as e:
                    if e.errno != errno.EEXIST:
                        raise
            else:
                raise

        # Fall through to here if stat succeeded or open raised EEXIST.
        raise mailbox.ExternalClashError('Name clash prevented file creation: %s' % path)


class QueueError(Exception):

    def __init__(self, msg, data):
        Exception.__init__(self, msg)
        self._message = msg
        self.data = data


class Queue(object):
    """
    Provides a simplified API for dealing with 'queues' in Salmon.
    It currently just supports Maildir queues since those are the
    most robust, but could implement others later.
    """

    def __init__(self, queue_dir, safe=False, pop_limit=0, oversize_dir=None):
        """
        This gives the Maildir queue directory to use, and whether you want
        this Queue to use the SafeMaildir variant which hashes the hostname
        so you can expose it publicly.

        The pop_limit and oversize_queue both set a upper limit on the mail
        you pop out of the queue.  The size is checked before any Salmon
        processing is done and is based on the size of the file on disk.  The
        purpose is to prevent people from sending 10MB attachments.  If a
        message is over the pop_limit then it is placed into the
        oversize_dir (which should be a Maildir).

        The oversize protection only works on pop messages off, not
        putting them in, get, or any other call.  If you use get you can
        use self.oversize to also check if it's oversize manually.
        """
        self.dir = queue_dir

        if safe:
            self.mbox = SafeMaildir(queue_dir)
        else:
            self.mbox = mailbox.Maildir(queue_dir)

        self.pop_limit = pop_limit

        if oversize_dir:
            if not os.path.exists(oversize_dir):
                mailbox.Maildir(oversize_dir)

            self.oversize_dir = os.path.join(oversize_dir, "new")

            if not os.path.exists(self.oversize_dir):
                os.mkdir(self.oversize_dir)
        else:
            self.oversize_dir = None

    def push(self, message):
        """
        Pushes the message onto the queue.  Remember the order is probably
        not maintained.  It returns the key that gets created.
        """
        if not isinstance(message, (six.text_type, six.binary_type)):
            message = str(message)
        return self.mbox.add(message)

    def pop(self):
        """
        Pops a message off the queue, order is not really maintained
        like a stack.

        It returns a (key, message) tuple for that item.
        """
        for key in self.mbox.iterkeys():
            over, over_name = self.oversize(key)

            if over:
                if self.oversize_dir:
                    logging.info("Message key %s over size limit %d, moving to %s.",
                                 key, self.pop_limit, self.oversize_dir)
                    os.rename(over_name, os.path.join(self.oversize_dir, key))
                else:
                    logging.info("Message key %s over size limit %d, DELETING (set oversize_dir).",
                                 key, self.pop_limit)
                    os.unlink(over_name)
            else:
                try:
                    msg = self.get(key)
                except QueueError as exc:
                    raise exc
                finally:
                    self.remove(key)
                return key, msg

        return None, None

    def get(self, key):
        """
        Get the specific message referenced by the key.  The message is NOT
        removed from the queue.
        """
        msg_file = self.mbox.get_file(key)

        if not msg_file:
            return None

        msg_data = msg_file.read()

        try:
            return mail.MailRequest(self.dir, None, None, msg_data)
        except Exception as exc:
            logging.exception("Failed to decode message: %s; msg_data: %r",   exc, msg_data)
            return None

    def remove(self, key):
        """Removes the queue, but not returned."""
        self.mbox.remove(key)

    def __len__(self):
        """Returns the number of messages in the queue."""
        return len(self.mbox)

    # synonym of __len__ for backwards compatibility
    count = __len__

    def clear(self):
        """
        Clears out the contents of the entire queue.

        Warning: This could be horribly inefficient since it pops messages
        until the queue is empty. It could also cause an infinite loop if
        another process is writing to messages to the Queue faster than we can
        pop.
        """
        # man this is probably a really bad idea
        while len(self) > 0:
            self.pop()

    def keys(self):
        """
        Returns the keys in the queue.
        """
        return self.mbox.keys()

    def oversize(self, key):
        if self.pop_limit:
            file_name = os.path.join(self.dir, "new", key)
            return os.path.getsize(file_name) > self.pop_limit, file_name
        else:
            return False, None



from __future__ import print_function

from salmon.routing import Router, nolocking, route, route_like, state_key_generator, stateless


@state_key_generator
def simple_key_gen(module_name, message):
    return module_name


# common routing capture regexes go in here, you can override them in @route
Router.defaults(host="localhost",
                action="[a-zA-Z0-9]+",
                list_name="[a-zA-Z.0-9]+")


@route("(list_name)-(action)@(host)")
def START(message, list_name=None, action=None, host=None):
    print("START", message, list_name, action, host)
    if action == 'explode':
        print("EXPLODE!")
        raise RuntimeError("Exploded on purpose.")
    return CONFIRM


@route("(list_name)-confirm-(id_number)@(host)", id_number="[0-9]+")
def CONFIRM(message, list_name=None, id_number=None, host=None):
    print("CONFIRM", message, list_name, id_number, host)
    return POSTING


@route("(list_name)-(action)@(host)")
def POSTING(message, list_name=None, action=None, host=None):
    print("POSTING", message, list_name, action, host)
    return NEXT


@route_like(POSTING)
def NEXT(message, list_name=None, action=None, host=None):
    print("NEXT", message, list_name, action, host)
    return END


@route("(anything)@(host)", anything=".*")
def END(message, anything=None, host=None):
    print("END", anything, host)
    return START


@route(".*")
@stateless
@nolocking
def PASSING(message, *args, **kw):
    print("PASSING", args, kw)


try:
    # `@stateless` needs to be "inside" `@route` or `@route_like`
    @stateless
    @route("badstateless@(host)")
    def BAD_STATELESS(message, *args, **kw):
        print("BAD_STATELESS", args, kw)
except AssertionError:
    pass
else:
    raise Exception("No assertion error raised on misordered decorators")



"""
Implements a handler that puts every message it receives into
the run/queue directory.  It is intended as a debug tool so you
can inspect messages the server is receiving using mutt or
the salmon queue command.
"""

import logging

from salmon import handlers, queue
from salmon.routing import nolocking, route_like, stateless


@route_like(handlers.log.START)
@stateless
@nolocking
def START(message, to=None, host=None):
    """
    @stateless and routes however handlers.log.START routes (everything).
    Has @nolocking, but that's alright since it's just writing to a Maildir.
    """
    logging.debug("MESSAGE to %s@%s added to queue.", to, host)
    q = queue.Queue('run/queue')
    q.push(message)



from salmon.routing import route, route_like


@route("(address)@(host)", address=".+")
def START(message, address=None, host=None):
    return NEW_USER


@route_like(START)
def NEW_USER(message, address=None, host=None):
    return NEW_USER


@route_like(START)
def END(message, address=None, host=None):
    return START



# This file contains python variables that configure Salmon for email processing.
relay_config = {'host': 'localhost', 'port': 8825}

receiver_config = {'host': 'localhost', 'port': 8823}

handlers = []

dump_handlers = ["test_handlers.dump"]

router_defaults = {'host': 'localhost'}

template_config = {'dir': 'tests', 'module': '.'}

# this is for when you run the config.queue boot
queue_config = {'queue': 'run/deferred', 'sleep': 10}

queue_handlers = []

QUEUE_PATH = "run/dump"
UNDELIVERABLE_QUEUE = "run/undeliverable"



"""
These are helper functions that make it easier to work with either Jinja2 or
Mako templates.  You MUST configure it by setting salmon.view.LOADER to one of
the template loaders in your boot module. Remember to install Jinja2 or Mako
first though!

After that these functions should just work.
"""
from __future__ import print_function, unicode_literals

from salmon import mail

LOADER = None


def load(template):
    """
    Uses the registered loader to load the template you ask for.
    It assumes that your loader works like Jinja2 or Mako in that
    it has a LOADER.get_template() method that returns the template.
    """
    assert LOADER, "You haven't set salmon.view.LOADER to a loader yet."
    return LOADER.get_template(template)


def render(variables, template):
    """
    Takes the variables given and renders the template for you.
    Assumes the template returned by load() will have a .render()
    method that takes the variables as a dict.

    Use this if you just want to render a single template and don't
    want it to be a message.  Use render_message if the contents
    of the template are to be interpreted as a message with headers
    and a body.
    """
    return load(template).render(variables)


def respond(variables, Body=None, Html=None, **kwd):
    """
    Does the grunt work of cooking up a MailResponse that's based
    on a template.  The only difference from the salmon.mail.MailResponse
    class and this (apart from variables passed to a template) are that
    instead of giving actual Body or Html parameters with contents,
    you give the name of a template to render.  The kwd variables are
    the remaining keyword arguments to MailResponse of From/To/Subject.

    For example, to render a template for the body and a .html for the Html
    attachment, and to indicate the From/To/Subject do this:

        msg = view.respond(locals(), Body='template.txt',
                          Html='template.html',
                          From='test@test.com',
                          To='receiver@test.com',
                          Subject='Test body from "%(person)s".')

    In this case you're using locals() to gather the variables needed for
    the 'template.txt' and 'template.html' templates.  Each template is
    setup to be a text/plain or text/html attachment.  The From/To/Subject
    are setup as needed.  Finally, the locals() are also available as
    simple Python keyword templates in the From/To/Subject so you can pass
    in variables to modify those when needed (as in the %(person)s in Subject).
    """

    assert Body or Html, "You need to give either the Body or Html template of the mail."

    for key in kwd:
        kwd[key] = kwd[key] % variables

    msg = mail.MailResponse(**kwd)

    if Body:
        msg.Body = render(variables, Body)

    if Html:
        msg.Html = render(variables, Html)

    return msg


def attach(msg, variables, template, filename=None, content_type=None,
           disposition=None):
    """
    Useful for rendering an attachment and then attaching it to the message
    given.  All the parameters that are in salmon.mail.MailResponse.attach
    are there as usual.
    """
    data = render(variables, template)

    msg.attach(filename=filename, data=data, content_type=content_type,
               disposition=disposition)



"""
The salmon.mail module contains nothing more than wrappers around the big work
done in salmon.encoding.  These are the actual APIs that you'll interact with
when doing email, and they mostly replicate the salmon.encoding.MailBase
functionality.

The main design criteria is that MailRequest is mostly for reading email
that you've received, so it doesn't have functions for attaching files and such.
MailResponse is used when you are going to write an email, so it has the
APIs for doing attachments and such.
"""
from __future__ import print_function, unicode_literals

from email.utils import parseaddr
import mimetypes
import os
import warnings

import six

from salmon import bounce, encoding

# You can change this to 'Delivered-To' on servers that support it like Postfix
ROUTABLE_TO_HEADER = 'to'


def _decode_header_randomness(addr):
    """
    This fixes the given address so that it is *always* a set() of
    just email addresses suitable for routing.
    """
    if not addr:
        return set()
    elif isinstance(addr, (list, tuple)):
        addr_set = set()
        for a in addr:
            for returned_addr in _decode_header_randomness(a):
                addr_set.add(returned_addr)

        return addr_set
    elif isinstance(addr, six.string_types):
        return set([parseaddr(addr.lower())[1]])
    elif isinstance(addr, six.binary_type):
        addr = addr.decode()
        return set([parseaddr(addr.lower())[1]])
    else:
        raise encoding.EncodingError("Address must be a string or a list not: %r", type(addr))


class MailRequest(object):
    """
    This is what is given to your message handlers. The information you get out
    of this is *ALWAYS* in Python str (unicode in Python 2.7) and should be
    usable by any API.  Modifying this object will cause other handlers that
    deal with it to get your modifications, but in general you don't want to do
    more than maybe tag a few headers.
    """
    def __init__(self, Peer, From, To, Data):
        """
        Peer is the remote peer making the connection (sometimes the queue
        name).  From and To are what you think they are.  Data is the raw
        full email as received by the server.

        NOTE:  It does not handle multiple From headers, if that's even
        possible.  It will parse the From into a list and take the first
        one.
        """
        self.Peer = Peer
        self.Data = Data
        try:
            self.From = _decode_header_randomness(From).pop()
        except KeyError:
            self.From = None
        try:
            self.To = _decode_header_randomness(To).pop()
        except KeyError:
            self.To = None

        self.base = encoding.from_string(self.Data)

        if 'from' not in self.base:
            self.base['from'] = self.From
        if 'to' not in self.base:
            # do NOT use ROUTABLE_TO here
            self.base['to'] = self.To

        self.From = self.From or self.base['from']
        self.To = self.To or self.base[ROUTABLE_TO_HEADER]

        self.bounce = None

    def __repr__(self):
        return "From: %r" % [self.Peer, self.From, self.To]

    def all_parts(self):
        """Returns all multipart mime parts.  This could be an empty list."""
        return self.base.parts

    def body(self):
        """
        Always returns a body if there is one.  If the message
        is multipart then it returns the first part's body, if
        it's not then it just returns the body.  If returns
        None then this message has nothing for a body.
        """
        if self.base.parts:
            return self.base.parts[0].body
        else:
            return self.base.body

    def __contains__(self, key):
        return self.base.__contains__(key)

    def __getitem__(self, name):
        return self.base.__getitem__(name)

    def __setitem__(self, name, val):
        self.base.__setitem__(name, val)

    def __delitem__(self, name):
        del self.base[name]

    def __str__(self):
        """
        Converts this to a string usable for storage into a queue or
        transmission.
        """
        return encoding.to_string(self.base)

    def items(self):
        return self.base.items()

    def keys(self):
        return self.base.keys()

    def to_message(self):
        """
        Converts this to a Python email message you can use to
        interact with the python mail APIs.
        """
        return encoding.to_message(self.base)

    def walk(self):
        """Recursively walks all attached parts and their children."""
        for x in self.base.walk():
            yield x

    def is_bounce(self, threshold=0.3):
        """
        Determines whether the message is a bounce message based on
        salmon.bounce.BounceAnalzyer given threshold.  0.3 is a good
        conservative base.
        """
        if not self.bounce:
            self.bounce = bounce.detect(self)

        return self.bounce.score > threshold

    @property
    def original(self):
        warnings.warn("MailRequest.original is deprecated, use MailRequest.Data instead",
                      category=DeprecationWarning, stacklevel=2)
        return self.Data


class MailResponse(object):
    """
    You are given MailResponse objects from the salmon.view methods, and
    whenever you want to generate an email to send to someone.  It has
    the same basic functionality as MailRequest, but it is designed to
    be written to, rather than read from (although you can do both).

    You can easily set a Body or Html during creation or after by
    passing it as __init__ parameters, or by setting those attributes.

    You can initially set the From, To, and Subject, but they are headers so
    use the dict notation to change them:  msg['From'] = 'joe@test.com'.

    The message is not fully crafted until right when you convert it with
    MailResponse.to_message.  This lets you change it and work with it, then
    send it out when it's ready.
    """
    def __init__(self, To=None, From=None, Subject=None, Body=None, Html=None):
        self.Body = Body
        self.Html = Html
        self.base = encoding.MailBase([('To', To), ('From', From), ('Subject', Subject)])
        self.multipart = self.Body and self.Html
        self.attachments = []

    def __contains__(self, key):
        return self.base.__contains__(key)

    def __getitem__(self, key):
        return self.base.__getitem__(key)

    def __setitem__(self, key, val):
        return self.base.__setitem__(key, val)

    def __delitem__(self, name):
        del self.base[name]

    def attach(self, filename=None, content_type=None, data=None, disposition=None):
        """
        Simplifies attaching files from disk or data as files.  To attach simple
        text simple give data and a content_type.  To attach a file, give the
        data/content_type/filename/disposition combination.

        For convenience, if you don't give data and only a filename, then it
        will read that file's contents when you call to_message() later.  If you
        give data and filename then it will assume you've filled data with what
        the file's contents are and filename is just the name to use.
        """
        assert filename or data, "You must give a filename or some data to attach."
        assert data or os.path.exists(filename), "File doesn't exist, and no data given."

        self.multipart = True

        if filename and not content_type:
            content_type, encoding = mimetypes.guess_type(filename)

        assert content_type, "No content type given, and couldn't guess from the filename: %r" % filename

        self.attachments.append({
            'filename': filename,
            'content_type': content_type,
            'data': data,
            'disposition': disposition,
        })

    def attach_part(self, part):
        """
        Attaches a raw MailBase part from a MailRequest (or anywhere)
        so that you can copy it over.
        """
        self.multipart = True

        self.attachments.append({'filename': None,
                                 'content_type': None,
                                 'data': None,
                                 'disposition': None,
                                 'part': part,
                                 })

    def attach_all_parts(self, mail_request):
        """
        Used for copying the attachment parts of a mail.MailRequest
        object for mailing lists that need to maintain attachments.
        """
        for part in mail_request.all_parts():
            self.attach_part(part)

    def clear(self):
        """
        Clears out the attachments so you can redo them.  Use this to keep the
        headers for a series of different messages with different attachments.
        """
        del self.attachments[:]
        del self.base.parts[:]
        self.multipart = False

    def update(self, message):
        """
        Used to easily set a bunch of headers from another dict like object.
        """
        for k in message.keys():
            self.base[k] = message[k]

    def __str__(self):
        """
        Converts to a string.
        """
        return self.to_message().as_string()

    def _encode_attachment(self, filename=None, content_type=None, data=None, disposition=None, part=None):
        """
        Used internally to take the attachments mentioned in self.attachments
        and do the actual encoding in a lazy way when you call to_message.
        """
        if part:
            self.base.parts.append(part)
        elif filename:
            if not data:
                data = open(filename).read()

            self.base.attach_file(filename, data, content_type, disposition or 'attachment')
        else:
            self.base.attach_text(data, content_type)

        ctype = self.base.content_encoding['Content-Type'][0]

        if ctype and not ctype.startswith('multipart'):
            self.base.content_encoding['Content-Type'] = ('multipart/mixed', {})

    def to_message(self):
        """
        Figures out all the required steps to finally craft the
        message you need and return it.  The resulting message
        is also available as a self.base attribute.

        What is returned is a Python email API message you can
        use with those APIs.  The self.base attribute is the raw
        salmon.encoding.MailBase.
        """
        del self.base.parts[:]

        if self.Body and self.Html:
            self.multipart = True
            self.base.content_encoding['Content-Type'] = ('multipart/alternative', {})

        if self.multipart:
            self.base.body = None
            if self.Body:
                self.base.attach_text(self.Body, 'text/plain')

            if self.Html:
                self.base.attach_text(self.Html, 'text/html')

            for args in self.attachments:
                self._encode_attachment(**args)

        elif self.Body:
            self.base.body = self.Body
            self.base.content_encoding['Content-Type'] = ('text/plain', {})

        elif self.Html:
            self.base.body = self.Html
            self.base.content_encoding['Content-Type'] = ('text/html', {})

        return encoding.to_message(self.base)

    def all_parts(self):
        """
        Returns all the encoded parts.  Only useful for debugging
        or inspecting after calling to_message().
        """
        return self.base.parts

    def items(self):
        return self.base.items()

    def keys(self):
        return self.base.keys()



# Copyright (C) 2008 Zed A. Shaw.  Licensed under the terms of the GPLv3.
from __future__ import print_function

from unittest import TestCase

from salmon import encoding, mail

sample_message = """From: somedude@localhost
To: somedude@localhost

Test
"""


class MessageTestCase(TestCase):
    def test_mail_request(self):
        # try with a half-assed message
        msg = mail.MailRequest("localhost", "zedfrom@localhost",
                               "zedto@localhost", "Fake body.")
        self.assertEqual(msg['to'], "zedto@localhost")
        self.assertEqual(msg['from'], "zedfrom@localhost")

        msg = mail.MailRequest("localhost", "somedude@localhost", ["somedude@localhost"], sample_message)
        self.assertEqual(msg.original, sample_message)

        self.assertEqual(msg['From'], "somedude@localhost")

        assert("From" in msg)
        del msg["From"]
        assert("From" not in msg)

        msg["From"] = "nobody@localhost"
        assert("From" in msg)
        self.assertEqual(msg["From"], "nobody@localhost")
        msg["From"] = "somebody@localhost"
        self.assertEqual(msg["From"], "somebody@localhost")
        self.assertEqual(msg.keys(), ["To", "From"])
        self.assertEqual(msg.items(), [("To", "somedude@localhost"), ("From", "somebody@localhost")])

        # appending headers
        msg.base.append_header("To", "nobody@example.com")
        self.assertEqual(msg["To"], "somedude@localhost")
        self.assertEqual(msg.keys(), ["To", "From", "To"])
        self.assertEqual(msg.items(), [("To", "somedude@localhost"), ("From", "somebody@localhost"),
                                       ("To", "nobody@example.com")])

        # validate that upper and lower case work for headers
        assert("FroM" in msg)
        assert("from" in msg)
        assert("From" in msg)
        self.assertEqual(msg['From'], msg['fRom'])
        self.assertEqual(msg['From'], msg['from'])
        self.assertEqual(msg['from'], msg['fRom'])

        # make sure repr runs
        assert repr(msg)

        assert str(msg)

    def test_mail_response_plain_text(self):
        sample = mail.MailResponse(
            To="receiver@localhost",
            Subject="Test message",
            From="sender@localhost",
            Body="Test from test_mail_response_plain_text.",
        )

        assert str(sample)
        return sample

    def test_mail_response_html(self):
        sample = mail.MailResponse(
            To="receiver@localhost",
            Subject="Test message",
            From="sender@localhost",
            Html="<html><body><p>From test_mail_response_html</p></body></html>",
        )

        assert str(sample)
        return sample

    def test_mail_response_html_and_plain_text(self):
        sample = mail.MailResponse(
            To="receiver@localhost",
            Subject="Test message",
            From="sender@localhost",
            Html="<html><body><p>Hi there.</p></body></html>",
            Body="Test from test_mail_response_html_and_plain_text.",
        )

        assert str(sample)
        return sample

    def test_mail_response_attachments(self):
        sample = mail.MailResponse(
            To="receiver@localhost",
            Subject="Test message",
            From="sender@localhost",
            Body="Test from test_mail_response_attachments.",
        )
        with open("./README.rst") as file_obj:
            readme_data = file_obj.read()

        with self.assertRaises(AssertionError):
            sample.attach(data=readme_data, disposition="inline")

        sample.attach(filename="./README.rst", content_type="text/plain", disposition="inline")
        self.assertEqual(len(sample.attachments), 1)
        assert sample.multipart

        msg = sample.to_message()
        self.assertEqual(len(msg.get_payload()), 2)

        sample.clear()
        self.assertEqual(len(sample.attachments), 0)
        assert not sample.multipart

        sample.attach(data=readme_data, filename="./README.rst", content_type="text/plain")

        msg = sample.to_message()
        self.assertEqual(len(msg.get_payload()), 2)
        sample.clear()

        sample.attach(data=readme_data, content_type="text/plain")
        msg = sample.to_message()
        self.assertEqual(len(msg.get_payload()), 2)

        assert str(sample)
        return sample

    def test_mail_request_attachments(self):
        sample = self.test_mail_response_attachments()
        data = str(sample)

        msg = mail.MailRequest("localhost", None, None, data)

        msg_parts = msg.all_parts()
        sample_parts = sample.all_parts()

        with open("./README.rst") as file_obj:
            readme = file_obj.read()

        assert msg_parts[0].body == sample_parts[0].body
        assert readme == msg_parts[1].body
        assert msg.body() == sample_parts[0].body

        # test that we get at least one message for messages without attachments
        sample = self.test_mail_response_plain_text()
        msg = mail.MailRequest("localhost", None, None, str(sample))
        msg_parts = msg.all_parts()
        self.assertEqual(len(msg_parts), 0)
        assert msg.body()

    def test_mail_response_mailing_list_headers(self):
        list_addr = "test.users@localhost"

        msg = mail.MailResponse(From='somedude@localhost', To=list_addr, Subject='subject', Body="Mailing list reply.")

        print(repr(msg))

        msg["Sender"] = list_addr
        msg["Reply-To"] = list_addr
        msg["List-Id"] = list_addr
        msg["Return-Path"] = list_addr
        msg["In-Reply-To"] = 'Message-Id-1231123'
        msg["References"] = 'Message-Id-838845854'
        msg["Precedence"] = 'list'

        data = str(msg)

        req = mail.MailRequest('localhost', 'somedude@localhost', list_addr, data)

        headers = ['Sender', 'Reply-To', 'List-Id', 'Return-Path', 'In-Reply-To', 'References', 'Precedence']
        for header in headers:
            assert msg[header] == req[header], "%s: %r != %r" % (header, msg[header], req[header])

        # try a delete
        del msg['Precedence']

    def test_mail_response_headers(self):
        msg = self.test_mail_response_plain_text()
        # validate that upper and lower case work for headers
        assert("FroM" in msg)
        assert("from" in msg)
        assert("From" in msg)
        self.assertEqual(msg['From'], msg['fRom'])
        self.assertEqual(msg['From'], msg['from'])
        self.assertEqual(msg['from'], msg['fRom'])

        self.assertEqual(msg.keys(), [i[0] for i in msg.items()])

    def test_walk(self):
        with open("tests/data/bounce.msg") as file_obj:
            bm = mail.MailRequest(None, None, None, file_obj.read())
        parts = [x for x in bm.walk()]

        assert parts
        self.assertEqual(len(parts), 6)

    def test_copy_parts(self):
        with open("tests/data/bounce.msg") as file_obj:
            bm = mail.MailRequest(None, None, None, file_obj.read())

        resp = mail.MailResponse(To=bm['to'], From=bm['from'], Subject=bm['subject'])

        resp.attach_all_parts(bm)

        resp = resp.to_message()
        bm = bm.to_message()

        self.assertEqual(len([x for x in bm.walk()]), len([x for x in resp.walk()]))

    def test_craft_from_sample(self):
        list_name = "test.list"
        user_full_address = "tester@localhost"

        sample = mail.MailResponse(
            To=list_name + "@localhost",
            From=user_full_address,
            Subject="Test message with attachments.",
            Body="The body as one attachment.",
        )
        sample.update({"Test": "update"})

        sample.attach(filename="tests/message_tests.py", content_type="text/plain", disposition="attachment")

        inmsg = mail.MailRequest("fakepeer", None, None, str(sample))
        assert "Test" in sample.keys()

        for part in inmsg.to_message().walk():
            assert part.get_payload(), "inmsg busted."

        outmsg = mail.MailResponse(To=inmsg['from'], From=inmsg['to'], Subject=inmsg['subject'])

        outmsg.attach_all_parts(inmsg)

        result = outmsg.to_message()

        for part in result.walk():
            assert part.get_payload(), "outmsg parts don't have payload."

    def test_to_from_works(self):
        msg = mail.MailRequest("fakepeer", "from@localhost", [u"<to1@localhost>", u"to2@localhost"], "")
        assert '<' not in msg.To, msg.To

        msg = mail.MailRequest("fakepeer", "from@localhost", [u"to1@localhost", u"to2@localhost"], "")
        assert '<' not in msg.To, msg.To

        msg = mail.MailRequest("fakepeer", "from@localhost", [u"to1@localhost", u"<to2@localhost>"], "")
        assert '<' not in msg.To, msg.To

        msg = mail.MailRequest("fakepeer", "from@localhost", [u"to1@localhost"], "")
        assert '<' not in msg.To, msg.To

        msg = mail.MailRequest("fakepeer", "from@localhost", [u"<to1@localhost>"], "")
        assert '<' not in msg.To, msg.To

    def test_decode_header_randomness(self):
        self.assertEqual(mail._decode_header_randomness(None), set())

        # with strings
        self.assertEqual(mail._decode_header_randomness(["z@localhost", '"Z A" <z@localhost>']),
                         set(["z@localhost", "z@localhost"]))
        self.assertEqual(mail._decode_header_randomness("z@localhost"), set(["z@localhost"]))

        # with bytes
        self.assertEqual(mail._decode_header_randomness([b"z@localhost", b'"Z A" <z@localhost>']),
                         set(["z@localhost", "z@localhost"]))
        self.assertEqual(mail._decode_header_randomness(b"z@localhost"), set(["z@localhost"]))

        # with literal nonsense
        with self.assertRaises(encoding.EncodingError):
            mail._decode_header_randomness(1)
        with self.assertRaises(encoding.EncodingError):
            mail._decode_header_randomness([1, "m@localhost"])



import logging.config

from salmon import queue
from salmon.routing import Router
from salmon.server import LMTPReceiver, Relay

from . import settings

logging.config.fileConfig("config/logging.conf")

# the relay host to actually send the final message to
settings.relay = Relay(host=settings.relay_config['host'],
                       port=settings.relay_config['port'], debug=1)

# where to listen for incoming messages
settings.receiver = LMTPReceiver(settings.receiver_config['host'],
                                 settings.receiver_config['port'])

Router.defaults(**settings.router_defaults)
Router.load(settings.handlers)
Router.RELOAD = True
Router.UNDELIVERABLE_QUEUE = queue.Queue("run/undeliverable")



from tempfile import mkdtemp
import mailbox
import os
import sys

from click import testing
from mock import Mock, patch

from salmon import queue, commands, encoding, mail, routing, utils

from .setup_env import SalmonTestCase


def make_fake_pid_file():
    with open("run/fake.pid", "w") as f:
        f.write("0")


class CliRunner(testing.CliRunner):
    def invoke(self, *args, **kwargs):
        kwargs.setdefault("catch_exceptions", False)
        return super(CliRunner, self).invoke(*args, **kwargs)


class CommandTestCase(SalmonTestCase):
    @patch("salmon.server.smtplib.SMTP")
    def test_send_command(self, client_mock):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("send", "--sender", "test@localhost", "--to", "test@localhost",
                                               "--body", "Test body", "--subject", "Test subject", "--attach",
                                               "setup.py", "--port", "8899", "--host", "127.0.0.1"))
        self.assertEqual(client_mock.return_value.sendmail.call_count, 1)
        self.assertEqual(result.exit_code, 0)

    def test_status_command(self):
        make_fake_pid_file()
        runner = CliRunner()
        running_result = runner.invoke(commands.main, ("status", "--pid", 'run/fake.pid'))
        self.assertEqual(running_result.output, "Salmon running with PID 0\n")
        self.assertEqual(running_result.exit_code, 0)

    def test_status_no_pid(self):
        runner = CliRunner()
        not_running_result = runner.invoke(commands.main, ("status", "--pid", 'run/donotexist.pid'))
        self.assertEqual(not_running_result.output, "Salmon not running.\n")
        self.assertEqual(not_running_result.exit_code, 1)

    def test_main(self):
        runner = CliRunner()
        result = runner.invoke(commands.main)
        self.assertEqual(result.exit_code, 0)

    @patch('salmon.queue.Queue')
    def test_queue_command(self, MockQueue):
        mq = MockQueue()
        mq.get.return_value = "A sample message"
        mq.keys.return_value = ["key1", "key2"]
        mq.pop.return_value = ('key1', 'message1')
        mq.__len__.return_value = 1

        runner = CliRunner()

        runner.invoke(commands.main, ("queue", "--pop"))
        self.assertEqual(mq.pop.call_count, 1)

        runner.invoke(commands.main, ("queue", "--get", "somekey"))
        self.assertEqual(mq.get.call_count, 1)

        runner.invoke(commands.main, ("queue", "--remove", "somekey"))
        self.assertEqual(mq.remove.call_count, 1)

        runner.invoke(commands.main, ("queue", "--clear"))
        self.assertEqual(mq.clear.call_count, 1)

        runner.invoke(commands.main, ("queue", "--keys"))
        self.assertEqual(mq.keys.call_count, 1)

        runner.invoke(commands.main, ("queue", "--count"))
        self.assertEqual(mq.__len__.call_count, 1)

    @patch('salmon.utils.daemonize')
    @patch('salmon.server.SMTPReceiver')
    def test_log_command(self, MockSMTPReceiver, daemon_mock):
        runner = CliRunner()
        ms = MockSMTPReceiver()
        ms.start.function()

        result = runner.invoke(commands.main, ("log", "--host", "127.0.0.1", "--port", "8825", "--pid", "run/fake.pid"))
        self.assertEqual(daemon_mock.call_count, 1)
        self.assertEqual(ms.start.call_count, 1)
        self.assertEqual(result.exit_code, 0)

        # test that it exits on existing pid
        make_fake_pid_file()
        result = runner.invoke(commands.main, ("log", "--host", "127.0.0.1", "--port", "8825", "--pid", "run/fake.pid"))
        self.assertEqual(result.exit_code, 1)

    @patch('sys.stdin', new=Mock())
    @patch("salmon.server.smtplib.SMTP")
    def test_sendmail_command(self, client_mock):
        sys.stdin.read.function()

        msg = mail.MailResponse(To="tests@localhost", From="tests@localhost",
                                Subject="Hello", Body="Test body.")
        sys.stdin.read.return_value = str(msg)

        runner = CliRunner()
        runner.invoke(commands.main, ("sendmail", "--host", "127.0.0.1", "--port", "8899", "test@localhost"))
        self.assertEqual(client_mock.return_value.sendmail.call_count, 1)


class StartCommandTestCase(SalmonTestCase):
    @patch('salmon.utils.daemonize')
    @patch('salmon.utils.import_settings')
    @patch('salmon.utils.drop_priv')
    def test_start_command(self, priv_mock, settings_mock, daemon_mock):
        runner = CliRunner()
        runner.invoke(commands.main, ("start", "--pid", "smtp.pid"))
        self.assertEqual(daemon_mock.call_count, 1)
        self.assertEqual(daemon_mock.call_args, (("smtp.pid", ".", None, None), {"files_preserve": []}))
        self.assertEqual(settings_mock.call_count, 1)
        self.assertEqual(settings_mock.call_args, ((True,), {"boot_module": "config.boot"}))

    @patch('salmon.utils.daemonize')
    @patch('salmon.utils.import_settings')
    def test_pid(self, settings_mock, daemon_mock):
        runner = CliRunner()
        # start with pid file existing already
        make_fake_pid_file()
        result = runner.invoke(commands.main, ("start", "--pid", "run/fake.pid"))
        self.assertEqual(result.exit_code, 1)
        self.assertEqual(daemon_mock.call_count, 0)

        # start with pid file existing and force given
        self.assertTrue(os.path.exists("run/fake.pid"))
        runner.invoke(commands.main, ("start", "--force", "--pid", "run/fake.pid"))
        self.assertFalse(os.path.exists("run/fake.pid"))
        self.assertEqual(daemon_mock.call_count, 1)
        self.assertEqual(daemon_mock.call_args, (("run/fake.pid", ".", None, None), {"files_preserve": []}))

    @patch('salmon.utils.daemonize')
    @patch('salmon.utils.import_settings')
    @patch('salmon.utils.drop_priv')
    def test_set_uid_and_guid(self, priv_mock, settings_mock, daemon_mock):
        runner = CliRunner()
        # start with a uid but no gid
        runner.invoke(commands.main, ("start", "--uid", "1000", "--pid", "run/fake.pid", "--force"))
        self.assertEqual(priv_mock.call_count, 0)
        self.assertEqual(daemon_mock.call_count, 1)
        self.assertEqual(daemon_mock.call_args, (("run/fake.pid", ".", None, None), {"files_preserve": []}))

        # start with a uid/gid given that's valid
        runner.invoke(commands.main, ("start", "--uid", "1000", "--gid", "1000", "--pid", "run/fake.pid", "--force"))
        self.assertEqual(priv_mock.call_count, 1)
        self.assertEqual(priv_mock.call_args, ((1000, 1000), {}))
        self.assertEqual(daemon_mock.call_count, 2)
        self.assertEqual(daemon_mock.call_args, (("run/fake.pid", ".", None, None), {"files_preserve": []}))

    @patch('salmon.utils.daemonize')
    @patch('salmon.utils.import_settings')
    def test_non_daemon(self, settings_mock, daemon_mock):
        runner = CliRunner()
        runner.invoke(commands.main, ("start", "--pid", "run/fake.pid", "--no-daemon", "--force"))
        self.assertEqual(daemon_mock.call_count, 0)


class CleanseCommandTestCase(SalmonTestCase):
    def setUp(self):
        super(CleanseCommandTestCase, self).setUp()
        queue.Queue("run/queue").clear()

    def test_cleanse_command(self):
        q = queue.Queue("run/queue")
        msg_count = 3
        for i in range(msg_count):
            msg = mail.MailResponse(To="tests%s@localhost" % i, From="tests%s@localhost" % i,
                                    Subject="Hello", Body="Test body.")
            q.push(msg)
        self.assertEqual(q.count(), msg_count)
        runner = CliRunner()
        result = runner.invoke(commands.main, ("cleanse", "run/queue", "run/cleansed"))
        self.assertEqual(result.exit_code, 0)
        self.assertEqual(q.count(), msg_count)
        outbox = mailbox.Maildir("run/cleansed", create=False)
        self.assertEqual(len(outbox), msg_count)

    def test_no_inbox(self):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("cleanse", "run/not-a-queue", "run/cleansed"))
        self.assertEqual(result.exit_code, 1)
        self.assertEqual(result.output, "Error: run/not-a-queue does not exist or is not a valid MBox or Maildir\n")
        self.assertFalse(os.path.exists('run/cleansed'))

    def test_empty(self):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("cleanse", "run/queue", "run/cleansed"))
        self.assertEqual(result.exit_code, 0)
        self.assertTrue(os.path.exists('run/cleansed'))

    @patch('salmon.encoding.from_message')
    def test_encoding_error(self, from_message):
        runner = CliRunner()
        from_message.side_effect = encoding.EncodingError
        in_queue = "run/queue"
        q = queue.Queue(in_queue)
        q.push("hello")

        result = runner.invoke(commands.main, ("cleanse", in_queue, "run/cleased"))
        self.assertEqual(result.exit_code, 1)


class GenCommandTestCase(SalmonTestCase):
    def setUp(self):
        super(GenCommandTestCase, self).setUp()
        tmp_dir = mkdtemp()
        self.project = os.path.join(tmp_dir, 'testproject')

    def test_gen_command(self):
        runner = CliRunner()

        result = runner.invoke(commands.main, ("gen", self.project))
        assert os.path.exists(self.project)
        self.assertEqual(result.exit_code, 0)

    def test_if_folder_exists(self):
        runner = CliRunner()
        os.mkdir(self.project)

        result = runner.invoke(commands.main, ("gen", self.project))
        self.assertEqual(result.exit_code, 1)

    def test_force(self):
        runner = CliRunner()

        # folder doesn't exist, but user has used --force anyway
        result = runner.invoke(commands.main, ("gen", self.project, "--force"))
        assert os.path.exists(self.project)
        self.assertEqual(result.exit_code, 0)

        # assert again, this time the folder exists
        result = runner.invoke(commands.main, ("gen", self.project, "--force"))
        assert os.path.exists(self.project)
        self.assertEqual(result.exit_code, 0)


class StopCommandTestCase(SalmonTestCase):
    def setUp(self):
        super(StopCommandTestCase, self).setUp()
        patcher = patch("os.kill")
        patcher.start()
        self.addCleanup(patcher.stop)

    def test_stop_command(self):
        runner = CliRunner()
        make_fake_pid_file()
        result = runner.invoke(commands.main, ("stop", "--pid", "run/fake.pid"))
        self.assertEqual(result.exit_code, 0)
        self.assertEqual(os.kill.call_count, 1)

    def test_stop_pid_doesnt_exist(self):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("stop", "--pid", "run/dontexit.pid"))
        self.assertEqual(result.exit_code, 1)
        self.assertEqual(os.kill.call_count, 0)

    @patch('glob.glob', new=lambda x: ['run/fake.pid'])
    def test_stop_all(self):
        runner = CliRunner()
        make_fake_pid_file()
        result = runner.invoke(commands.main, ("stop", "--all", "run"))
        self.assertEqual(result.exit_code, 0)
        self.assertEqual(os.kill.call_count, 1)

    def test_stop_force(self):
        runner = CliRunner()
        make_fake_pid_file()
        result = runner.invoke(commands.main, ("stop", "--pid", "run/fake.pid", "--force"))
        self.assertEqual(os.kill.call_count, 1)
        self.assertEqual(result.exit_code, 0)
        assert not os.path.exists("run/fake.pid")

    def test_stop_force_oserror(self):
        runner = CliRunner()
        make_fake_pid_file()
        os.kill.side_effect = OSError("Fail")
        result = runner.invoke(commands.main, ("stop", "--pid", "run/fake.pid", "--force"))
        self.assertEqual(result.exit_code, 1)
        self.assertEqual(os.kill.call_count, 1)


class RoutesCommandTestCase(SalmonTestCase):
    def setUp(self):
        super(RoutesCommandTestCase, self).setUp()
        if "salmon.handlers.log" in sys.modules:
            del sys.modules["salmon.handlers.log"]
        routing.Router.clear_routes()
        routing.Router.clear_states()
        routing.Router.HANDLERS.clear()
        utils.settings = None

    def test_no_args(self):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("routes",))
        self.assertEqual(result.exit_code, 2)

    def test_not_importable(self):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("routes", "not_a_module", "--test", "user@example.com"))
        self.assertEqual(result.exit_code, 1)
        self.assertIsNotNone(utils.settings)
        self.assertEqual(result.output,
                         ("Error: Module 'not_a_module' could not be imported. "
                          "Did you forget to use the --path option?\n"))

    def test_match(self):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("routes", "salmon.handlers.log", "--test", "user@example.com"))
        self.assertEqual(result.exit_code, 0)
        self.assertIsNotNone(utils.settings)
        # TODO: use groupdict directly once Python 2.7 support has been dropped
        match_items = [i for i in
                       routing.Router.REGISTERED.values()][0][0].match("user@example.com").groupdict().items()
        self.assertEqual(result.output,
                         ("Routing ORDER: ['^(?P<to>.+)@(?P<host>.+)$']\n"
                          "Routing TABLE:\n"
                          "---\n"
                          "'^(?P<to>.+)@(?P<host>.+)$': salmon.handlers.log.START \n"
                          "---\n"
                          "\n"
                          "TEST address 'user@example.com' matches:\n"
                          "  '^(?P<to>.+)@(?P<host>.+)$' salmon.handlers.log.START\n"
                          "  -  %r\n" % {str(k): str(v) for k, v in match_items}))

    def test_no_match(self):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("routes", "salmon.handlers.log", "--test", "userexample.com"))
        self.assertEqual(result.exit_code, 1)
        self.assertIsNotNone(utils.settings)
        self.assertEqual(result.output,
                         ("Routing ORDER: ['^(?P<to>.+)@(?P<host>.+)$']\n"
                          "Routing TABLE:\n"
                          "---\n"
                          "'^(?P<to>.+)@(?P<host>.+)$': salmon.handlers.log.START \n"
                          "---\n"
                          "\n"
                          "TEST address 'userexample.com' didn't match anything.\n"))

    def test_no_test(self):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("routes", "salmon.handlers.log"))
        self.assertEqual(result.exit_code, 0)
        self.assertIsNotNone(utils.settings)
        self.assertEqual(result.output,
                         ("Routing ORDER: ['^(?P<to>.+)@(?P<host>.+)$']\n"
                          "Routing TABLE:\n"
                          "---\n"
                          "'^(?P<to>.+)@(?P<host>.+)$': salmon.handlers.log.START \n"
                          "---\n"))


class BlastCommandTestCase(SalmonTestCase):
    def setUp(self):
        super(BlastCommandTestCase, self).setUp()
        queue.Queue("run/queue").clear()

    @patch("salmon.server.smtplib.SMTP")
    def test_blast_command_empty(self, client_mock):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("blast", "--host", "129.0.0.1", "--port", "8899", "run/queue"))
        self.assertEqual(result.exit_code, 0)
        self.assertEqual(client_mock.call_count, 0)
        self.assertEqual(client_mock.return_value.sendmail.call_count, 0)

    @patch("salmon.server.smtplib.SMTP")
    def test_blast_three_messages(self, client_mock):
        q = queue.Queue("run/queue")
        msg_count = 3
        for i in range(msg_count):
            msg = mail.MailResponse(To="tests%s@localhost" % i, From="tests%s@localhost" % i,
                                    Subject="Hello", Body="Test body.")
            q.push(msg)
        runner = CliRunner()
        result = runner.invoke(commands.main, ("blast", "--host", "127.0.0.2", "--port", "8900", "run/queue"))
        self.assertEqual(result.exit_code, 0)
        self.assertEqual(client_mock.call_count, msg_count)
        self.assertEqual(client_mock.call_args_list, [
            (("127.0.0.2", 8900), {}),
            (("127.0.0.2", 8900), {}),
            (("127.0.0.2", 8900), {}),
        ])
        self.assertEqual(client_mock.return_value.sendmail.call_count, msg_count)

    def test_no_connection(self):
        q = queue.Queue("run/queue")
        msg = mail.MailResponse(To="tests@localhost", From="tests@localhost",
                                Subject="Hello", Body="Test body.")
        q.push(msg)
        runner = CliRunner()
        result = runner.invoke(commands.main, ("blast", "--host", "127.0.1.2", "--port", "8901", "run/queue"))
        self.assertEqual(result.exit_code, 1)
        self.assertEqual(result.output, "Error: [Errno 111] Connection refused\n")

    def test_no_queue(self):
        runner = CliRunner()
        result = runner.invoke(commands.main, ("blast", "--host", "127.1.1.2", "--port", "8889", "run/not-a-queue"))
        self.assertEqual(result.exit_code, 1)
        self.assertEqual(result.output, "Error: run/not-a-queue does not exist or is not a valid MBox or Maildir\n")



import mailbox
import os
import shutil

from mock import Mock, patch

from salmon import mail, queue

from .setup_env import SalmonTestCase

BYTES_MESSAGE = u"""From: me@localhost
To: you@localhost
Subject: bob!

Blobcat""".encode()


class QueueTestCase(SalmonTestCase):
    use_safe = False

    @classmethod
    def setUpClass(cls):
        shutil.rmtree("run/big_queue", ignore_errors=True)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree("run/big_queue", ignore_errors=True)

    def test_push(self):
        q = queue.Queue("run/queue", safe=self.use_safe)
        q.clear()

        # the queue doesn't really care if its a request or response, as long
        # as the object answers to str(msg)
        msg = mail.MailResponse(To="test@localhost", From="test@localhost", Subject="Test", Body="Test")
        key = q.push(msg)

        assert key, "Didn't get a key for test_get push."

        return q

    def test_pop(self):
        q = self.test_push()
        key, msg = q.pop()

        assert key, "Didn't get a key for test_get push."
        assert msg, "Didn't get a message for key %r" % key

        assert hasattr(msg, "Data"), "MailRequest doesn't have Data attribute"

        self.assertEqual(msg['to'], "test@localhost")
        self.assertEqual(msg['from'], "test@localhost")
        self.assertEqual(msg['subject'], "Test")
        self.assertEqual(msg.body(), "Test")

        self.assertEqual(q.count(), 0)
        assert not q.pop()[0]

    def test_get(self):
        q = self.test_push()
        msg = mail.MailResponse(To="test@localhost", From="test@localhost", Subject="Test", Body="Test")

        key = q.push(str(msg))
        assert key, "Didn't get a key for test_get push."

        msg = q.get(key)
        assert msg, "Didn't get a message for key %r" % key

    def test_remove(self):
        q = self.test_push()
        msg = mail.MailResponse(To="test@localhost", From="test@localhost", Subject="Test", Body="Test")

        key = q.push(str(msg))
        assert key, "Didn't get a key for test_get push."
        self.assertEqual(q.count(), 2)

        q.remove(key)
        self.assertEqual(q.count(), 1)

    def test_safe_maildir(self):
        self.use_safe = True
        self.test_push()
        self.test_pop()
        self.test_get()
        self.test_remove()

    def test_oversize_protections(self):
        # first just make an oversize limited queue
        overq = queue.Queue("run/queue", pop_limit=10)
        overq.clear()

        for i in range(5):
            overq.push("HELLO" * 100)

        self.assertEqual(overq.count(), 5)

        key, msg = overq.pop()

        assert not key and not msg, "Should get no messages."
        self.assertEqual(overq.count(), 0)

        # now make sure that oversize mail is moved to the overq
        overq = queue.Queue("run/queue", pop_limit=10, oversize_dir="run/big_queue")
        moveq = queue.Queue("run/big_queue")
        moveq.clear()

        for i in range(5):
            overq.push("HELLO" * 100)

        key, msg = overq.pop()

        assert not key and not msg, "Should get no messages."
        self.assertEqual(overq.count(), 0)
        self.assertEqual(moveq.count(), 5)

        moveq.clear()
        overq.clear()

    @patch('os.stat', new=Mock())
    def test_SafeMaildir_name_clash(self):
        sq = queue.SafeMaildir('run/queue')
        with self.assertRaises(mailbox.ExternalClashError):
            sq.add("TEST")

    @patch('mailbox._create_carefully', new=Mock())
    def test_SafeMaildir_throws_errno_failure(self):
        mailbox._create_carefully.side_effect = OSError
        sq = queue.SafeMaildir('run/queue')
        with self.assertRaises(OSError):
            sq.add('TEST')

    @patch('os.stat', new=Mock())
    def test_SafeMaildir_reraise_weird_errno(self):
        os.stat.side_effect = OSError
        sq = queue.SafeMaildir('run/queue')
        with self.assertRaises(OSError):
            sq.add('TEST')

    def test_bytes(self):
        """Test that passing a queue raw data works, i.e. as happens in the
        undeliverable queue"""
        q = queue.Queue("run/queue", safe=self.use_safe)
        q.clear()

        key = q.push(BYTES_MESSAGE)
        assert key, "Didn't get a key"

        mail = q.get(key)

        assert mail is not None, "Failed to get email from queue"

        self.assertEqual(mail['from'], "me@localhost")
        self.assertEqual(mail['to'], "you@localhost")
        self.assertEqual(mail['subject'], "bob!")
        self.assertEqual(mail.body(), "Blobcat")

    def test_len(self):
        q = self.test_push()
        self.assertEqual(len(q), 1)

    def test_count(self):
        q = self.test_push()
        self.assertEqual(q.count(), 1)



from salmon.bounce import bounce_to
from salmon.routing import route, route_like

SOFT_RAN = False
HARD_RAN = False


@route(".+")
def SOFT_BOUNCED(message):
    global SOFT_RAN
    SOFT_RAN = True
    # remember to transition back to START or the mailer daemon
    # at that host will be put in a bad state
    return START


@route(".+")
def HARD_BOUNCED(message):
    global HARD_RAN
    HARD_RAN = True
    # remember to transition back to START or the mailer daemon
    # at that host will be put in a bad state
    return START


@route("(anything)@(host)", anything=".+", host=".+")
@bounce_to(soft=SOFT_BOUNCED, hard=HARD_BOUNCED)
def START(message, **kw):
    return END


@route_like(START)
def END(message, *kw):
    pass



from setuptools import setup

import versioneer

install_requires = [
    'chardet',
    'click',
    'dnspython',
    'lmtpd>=4,<7.0.0',
    'python-daemon>2.2.0',
    'six',
]

tests_require = [
    'jinja2',
    'mock',
]

extras_require = {
    "docs": [
        "sphinx",
        "sphinx_rtd_theme",
        "sphinx-click",
    ],
}

config = {
    'description': 'A Python mail server forked from Lamson',
    'long_description': open("README.rst").read(),
    'url': 'https://github.com/moggers87/salmon',
    'download_url': 'http://pypi.python.org/pypi/salmon-mail',
    'author': 'Zed A. Shaw',
    'maintainer': 'Matt Molyneaux',
    'maintainer_email': 'moggers87+git@moggers87.co.uk',
    'version': versioneer.get_version(),
    'cmdclass': versioneer.get_cmdclass(),
    'install_requires': install_requires,
    'tests_require': tests_require,
    'extras_require': extras_require,
    'test_suite': 'tests',
    'packages': ['salmon', 'salmon.handlers'],
    'include_package_data': True,
    'name': 'salmon-mail',
    'license': 'GPLv3',
    'classifiers': [
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Intended Audience :: Developers',
        'Topic :: Communications :: Email',
        'Topic :: Software Development :: Libraries :: Application Frameworks'
        ],
    'entry_points': {
        'console_scripts':
            ['salmon = salmon.commands:main'],
    },
}

setup(**config)



# Copyright (C) 2008 Zed A. Shaw.  Licensed under the terms of the GPLv3.
from __future__ import print_function

import socket
import sys

from mock import Mock, call, patch
import lmtpd
import six

from salmon import mail, queue, routing, server

from .setup_env import SalmonTestCase

SMTP_MESSAGE_DEFS = {
    2: {"ok": u"250 Ok\r\n".encode()},
    3: {"ok": u"250 OK\r\n".encode()},
}

SMTP_MESSAGES = SMTP_MESSAGE_DEFS[sys.version_info[0]]


def generate_mail(factory=mail.MailRequest, peer="localhost", From="from@localhost", To="to@localhost",
                  Data="body", attachment=False):
    msg = factory(peer, From, To, Data)
    if attachment:
        msg.attach("./README.rst", content_type="text/plain", disposition="inline")
    return msg


class ServerTestCase(SalmonTestCase):
    def test_router(self):
        routing.Router.deliver(generate_mail())

        # test that fallthrough works too
        msg = generate_mail()
        msg['to'] = 'unhandled@localhost'
        msg.To = msg['to']

        routing.Router.deliver(msg)

    @patch("asynchat.async_chat.push")
    def test_SMTPChannel(self, push_mock):
        channel = server.SMTPChannel(Mock(), Mock(), Mock())
        expected_version = u"220 {} {}\r\n".format(socket.getfqdn(), server.smtpd.__version__).encode()

        self.assertEqual(push_mock.call_args[0][1:], (expected_version,))
        self.assertEqual(type(push_mock.call_args[0][1]), six.binary_type)

        push_mock.reset_mock()
        channel.seen_greeting = True
        channel.smtp_MAIL("FROM: you@example.com\r\n")
        self.assertEqual(push_mock.call_args[0][1:], (SMTP_MESSAGES["ok"],))

        push_mock.reset_mock()
        channel.smtp_RCPT("TO: me@example.com")
        self.assertEqual(push_mock.call_args[0][1:], (SMTP_MESSAGES["ok"],))

        push_mock.reset_mock()
        channel.smtp_RCPT("TO: them@example.com")
        self.assertEqual(push_mock.call_args[0][1:],
                         (u"451 Will not accept multiple recipients in one transaction\r\n".encode(),))

    def test_SMTPReceiver_process_message(self):
        receiver = server.SMTPReceiver(host="localhost", port=0)
        msg = generate_mail()

        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            response = receiver.process_message(msg.Peer, msg.From, msg.To, str(msg))
            assert response is None, response

        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            router_mock.deliver.side_effect = Exception()
            response = receiver.process_message(msg.Peer, msg.From, msg.To, str(msg))
            assert response is None, response

        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            router_mock.deliver.side_effect = server.SMTPError(450, "Not found")
            response = receiver.process_message(msg.Peer, msg.From, msg.To, str(msg))
            self.assertEqual(response, "450 Not found")

        # Python 3's smtpd takes some extra kawrgs, but i we don't care about that at the moment
        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            response = receiver.process_message(msg.Peer, msg.From, msg.To, str(msg), mail_options=[], rcpt_options=[])
            assert response is None, response

    @patch("lmtpd.asynchat.async_chat.push")
    def test_LMTPChannel(self, push_mock):
        channel = lmtpd.LMTPChannel(Mock(), Mock(), Mock())
        expected_version = u"220 {} {}\r\n".format(socket.getfqdn(), server.lmtpd.__version__).encode()

        self.assertEqual(push_mock.call_args[0][1:], (expected_version,))
        self.assertEqual(type(push_mock.call_args[0][1]), six.binary_type)

        push_mock.reset_mock()
        channel.seen_greeting = True
        channel.lmtp_MAIL(b"FROM: you@example.com\r\n")
        self.assertEqual(push_mock.call_args[0][1:], (u"250 2.1.0 Ok\r\n".encode(),))

        push_mock.reset_mock()
        channel.lmtp_RCPT(b"TO: me@example.com")
        self.assertEqual(push_mock.call_args[0][1:], (u"250 2.1.0 Ok\r\n".encode(),))

        push_mock.reset_mock()
        channel.lmtp_RCPT(b"TO: them@example.com")
        self.assertEqual(push_mock.call_args[0][1:], (u"250 2.1.0 Ok\r\n".encode(),))

    def test_LMTPReceiver_process_message(self):
        receiver = server.LMTPReceiver(host="localhost", port=0)
        msg = generate_mail()

        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            response = receiver.process_message(msg.Peer, msg.From, msg.To, str(msg))
            assert response is None, response

        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            router_mock.deliver.side_effect = Exception()
            response = receiver.process_message(msg.Peer, msg.From, msg.To, str(msg))
            assert response is None, response

        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            router_mock.deliver.side_effect = server.SMTPError(450, "Not found")
            response = receiver.process_message(msg.Peer, msg.From, msg.To, str(msg))
            self.assertEqual(response, "450 Not found")

        # lmtpd's server is a subclass of smtpd's server, so we should support the same kwargs here
        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            response = receiver.process_message(msg.Peer, msg.From, msg.To, str(msg), mail_options=[], rcpt_options=[])
            assert response is None, response

    @patch("salmon.queue.Queue")
    def test_QueueReceiver_process_message(self, queue_mock):
        receiver = server.QueueReceiver("run/queue/thingy")
        msg = generate_mail()

        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            response = receiver.process_message(msg)
            assert response is None, response

        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            router_mock.deliver.side_effect = Exception()
            response = receiver.process_message(msg)
            assert response is None, response

        with patch("salmon.server.routing.Router") as router_mock, \
                patch("salmon.server.undeliverable_message"):
            router_mock.deliver.side_effect = server.SMTPError(450, "Not found")
            response = receiver.process_message(msg)
            # queue doesn't actually support SMTPErrors
            assert response is None, response

    def test_Relay_asserts_ssl_options(self):
        """Relay raises an AssertionError if the ssl option is used in combination with starttls or lmtp"""
        with self.assertRaises(AssertionError):
            server.Relay("localhost", ssl=True, starttls=True)

        with self.assertRaises(AssertionError):
            server.Relay("localhost", ssl=True, lmtp=True)

        with self.assertRaises(AssertionError):
            server.Relay("localhost", ssl=True, starttls=True, lmtp=True)

        # no error
        server.Relay("localhost", starttls=True, lmtp=True)

    @patch("salmon.server.smtplib.SMTP")
    def test_relay_deliver(self, client_mock):
        # test that relay will actually call smtplib.SMTP
        relay = server.Relay("localhost", port=0)

        msg = generate_mail(factory=mail.MailResponse, attachment=True)
        relay.deliver(msg)
        self.assertEqual(client_mock.return_value.sendmail.call_count, 1)

        msg = generate_mail(factory=mail.MailResponse, attachment=True)
        relay.deliver(msg)
        self.assertEqual(client_mock.return_value.sendmail.call_count, 2)

        msg = generate_mail(factory=mail.MailResponse, attachment=True)
        relay.deliver(msg)
        self.assertEqual(client_mock.return_value.sendmail.call_count, 3)

        msg = generate_mail(factory=mail.MailResponse, attachment=True)
        relay.deliver(msg)
        self.assertEqual(client_mock.return_value.sendmail.call_count, 4)

    @patch("salmon.server.smtplib.SMTP")
    def test_relay_smtp(self, client_mock):
        relay = server.Relay("localhost", port=0)
        relay.deliver(generate_mail(factory=mail.MailResponse, attachment=True))
        self.assertEqual(client_mock.return_value.sendmail.call_count, 1)
        self.assertEqual(client_mock.return_value.starttls.call_count, 0)

        client_mock.reset_mock()
        relay = server.Relay("localhost", port=0, starttls=True)
        relay.deliver(generate_mail(factory=mail.MailResponse, attachment=True))
        self.assertEqual(client_mock.return_value.sendmail.call_count, 1)
        self.assertEqual(client_mock.return_value.starttls.call_count, 1)

    @patch("salmon.server.smtplib.LMTP")
    def test_relay_lmtp(self, client_mock):
        relay = server.Relay("localhost", port=0, lmtp=True)
        relay.deliver(generate_mail(factory=mail.MailResponse, attachment=True))
        self.assertEqual(client_mock.return_value.sendmail.call_count, 1)

    @patch("salmon.server.smtplib.SMTP_SSL")
    def test_relay_smtp_ssl(self, client_mock):
        relay = server.Relay("localhost", port=0, ssl=True)
        relay.deliver(generate_mail(factory=mail.MailResponse, attachment=True))
        self.assertEqual(client_mock.return_value.sendmail.call_count, 1)

    @patch('salmon.server.resolver.query')
    @patch("salmon.server.smtplib.SMTP")
    def test_relay_deliver_mx_hosts(self, client_mock, query):
        query.return_value = [Mock()]
        query.return_value[0].exchange = "localhost"
        relay = server.Relay(None, port=0)

        msg = generate_mail(factory=mail.MailResponse, attachment=True)
        msg['to'] = 'user@localhost'
        relay.deliver(msg)
        self.assertEqual(query.call_count, 1)

    @patch('salmon.server.resolver.query')
    def test_relay_resolve_relay_host(self, query):
        from dns import resolver
        query.side_effect = resolver.NoAnswer
        relay = server.Relay(None, port=0)
        host = relay.resolve_relay_host('user@localhost')
        self.assertEqual(host, 'localhost')
        self.assertEqual(query.call_count, 1)

        query.reset_mock()
        query.side_effect = None  # reset_mock doens't clear return_value or side_effect
        query.return_value = [Mock()]
        query.return_value[0].exchange = "mx.example.com"
        host = relay.resolve_relay_host('user@example.com')
        self.assertEqual(host, 'mx.example.com')
        self.assertEqual(query.call_count, 1)

    @patch("salmon.server.smtplib.SMTP")
    def test_relay_reply(self, client_mock):
        relay = server.Relay("localhost", port=0)
        print("Relay: %r" % relay)

        relay.reply(generate_mail(), 'from@localhost', 'Test subject', 'Body')
        self.assertEqual(client_mock.return_value.sendmail.call_count, 1)

    @patch("socket.create_connection")
    def test_relay_raises_exception(self, create_mock):
        # previously, salmon would eat up socket errors and just log something. Not cool!
        create_mock.side_effect = socket.error
        relay = server.Relay("example.com", port=0)
        with self.assertRaises(socket.error):
            relay.deliver(generate_mail(factory=mail.MailResponse))

    @patch('salmon.routing.Router')
    def test_queue_receiver(self, router_mock):
        receiver = server.QueueReceiver('run/queue')
        run_queue = queue.Queue('run/queue')
        run_queue.push(str(generate_mail(factory=mail.MailResponse)))
        assert run_queue.count() > 0
        receiver.start(one_shot=True)
        self.assertEqual(run_queue.count(), 0)
        self.assertEqual(run_queue.count(), 0)
        self.assertEqual(router_mock.deliver.call_count, 1)

        router_mock.deliver.side_effect = RuntimeError("Raised on purpose")
        receiver.process_message(mail.MailRequest('localhost', 'test@localhost', 'test@localhost', 'Fake body.'))

    @patch('salmon.routing.Router')
    @patch("salmon.server.queue.Queue")
    def test_queue_receiver_pop_error(self, queue_mock, router_mock):
        def key_error(*args, **kwargs):
            queue_mock.return_value.__len__.return_value = 0
            raise KeyError

        queue_mock.return_value.__len__.return_value = 1
        queue_mock.return_value.pop.side_effect = key_error
        receiver = server.QueueReceiver('run/queue')
        receiver.start(one_shot=True)
        self.assertEqual(queue_mock.return_value.pop.call_count, 1)
        self.assertEqual(router_mock.deliver.call_count, 0)

    @patch("salmon.server.time.sleep")
    @patch("salmon.server.Pool")
    def test_queue_receiver_sleep(self, pool_mock, sleep_mock):
        class SleepCalled(Exception):
            pass

        def sleepy(*args, **kwargs):
            if sleep_mock.call_count > 1:
                raise SleepCalled()

        sleep_mock.side_effect = sleepy

        receiver = server.QueueReceiver('run/queue', sleep=10, workers=1)
        with self.assertRaises(SleepCalled):
            receiver.start()

        self.assertEqual(receiver.workers.apply_async.call_count, 0)
        self.assertEqual(sleep_mock.call_count, 2)
        self.assertEqual(sleep_mock.call_args_list, [call(receiver.sleep), call(receiver.sleep)])

    @patch("salmon.server.Pool")
    def test_queue_receiver_pool(self, pool_mock):
        run_queue = queue.Queue('run/queue')
        msg = str(generate_mail(factory=mail.MailResponse))
        run_queue.push(msg)

        receiver = server.QueueReceiver('run/queue', sleep=10, workers=1)
        receiver.start(one_shot=True)

        self.assertEqual(receiver.workers.apply_async.call_count, 1)
        self.assertEqual(receiver.workers.apply_async.call_args[0], (receiver.process_message,))

        args = receiver.workers.apply_async.call_args[1]["args"]
        del receiver.workers.apply_async.call_args[1]["args"]

        # onlly the "args" kwarg should be present
        self.assertEqual(receiver.workers.apply_async.call_args[1], {})

        # we can't compare two Mail* objects, so we'll just check the type
        self.assertEqual(len(args), 1)
        self.assertEqual(type(args[0]), mail.MailRequest)

    @patch('threading.Thread', new=Mock())
    @patch('salmon.routing.Router', new=Mock())
    def test_SMTPReceiver(self):
        receiver = server.SMTPReceiver(port=0)
        receiver.start()
        receiver.process_message('localhost', 'test@localhost', 'test@localhost',
                                 'Fake body.')

        routing.Router.deliver.side_effect = RuntimeError("Raised on purpose")
        receiver.process_message('localhost', 'test@localhost', 'test@localhost', 'Fake body.')

        receiver.close()

    def test_SMTPError(self):
        err = server.SMTPError(550)
        self.assertEqual(str(err), '550 Permanent Failure Mail Delivery Protocol Status')

        err = server.SMTPError(400)
        self.assertEqual(str(err), '400 Persistent Transient Failure Other or Undefined Status')

        err = server.SMTPError(425)
        self.assertEqual(str(err), '425 Persistent Transient Failure Mailbox Status')

        err = server.SMTPError(999)
        self.assertEqual(str(err), "999 ")

        err = server.SMTPError(999, "Bogus Error Code")
        self.assertEqual(str(err), "999 Bogus Error Code")



from mock import Mock, patch

from salmon.testing import RouterConversation, assert_in_state, clear_queue, delivered, queue, relay

from .setup_env import SalmonTestCase


class TestingTestCase(SalmonTestCase):
    def test_clear_queue(self):
        queue().push("Test")
        self.assertEqual(queue().count(), 1)

        clear_queue()
        self.assertEqual(queue().count(), 0)

    @patch("smtplib.SMTP")
    def test_relay(self, smtp_mock):
        relay_obj = relay(port=0)
        smtp_mock.return_value = Mock()

        relay_obj.send('test@localhost', 'zedshaw@localhost', 'Test message', 'Test body')

        self.assertEqual(smtp_mock.return_value.sendmail.call_count, 1)
        self.assertEqual(smtp_mock.return_value.quit.call_count, 1)

    def test_delivered(self):
        clear_queue()
        queue().push("To: gooduser@localhost\nFrom: tester@localhost\n\nHi\n")

        assert delivered("gooduser@localhost"), "Test message not delivered."
        assert not delivered("baduser@localhost")
        assert_in_state('tests.handlers.simple_fsm_mod', 'gooduser@localhost', 'tester@localhost', 'START')

    def test_RouterConversation(self):
        client = RouterConversation('tester@localhost', 'Test router conversations.')
        client.begin()
        client.say('testlist@localhost', 'This is a test')
        delivered('testlist@localhost'), "Test message not delivered."



from tempfile import mkdtemp
from unittest import TestCase
import os

from mock import patch

from salmon import utils

from .setup_env import SalmonTestCase


class UtilsTestCase(SalmonTestCase):
    def tearDown(self):
        super(UtilsTestCase, self).tearDown()
        self.clear_settings()

    def clear_settings(self):
        utils.settings = None

    def test_make_fake_settings(self):
        settings = utils.make_fake_settings('localhost', 8800)
        assert settings
        assert settings.receiver
        assert settings.relay is None
        settings.receiver.close()

    def test_import_settings(self):
        assert utils.settings is None

        settings = utils.import_settings(True, boot_module='config.testing')
        assert settings
        assert settings.receiver_config
        self.assertEqual(settings, utils.settings)

        with patch("salmon.utils.importlib.import_module") as import_mock:
            # just import settings module
            self.clear_settings()
            utils.import_settings(False)
            self.assertEqual(import_mock.call_count, 1)

            # import settings and boot
            self.clear_settings()
            import_mock.reset_mock()
            utils.import_settings(True)
            self.assertEqual(import_mock.call_count, 2)

            # settings has already been imported, return early
            import_mock.reset_mock()
            utils.import_settings(False)
            self.assertEqual(import_mock.call_count, 0)

            # boot module doesn't get cached on the module, so it import_module should be called
            import_mock.reset_mock()
            utils.import_settings(True)
            self.assertEqual(import_mock.call_count, 1)

    @patch("daemon.daemon.change_process_owner")
    def test_drop_priv(self, cpo):
        utils.drop_priv(100, 100)
        self.assertEqual(cpo.call_count, 1)
        self.assertEqual(cpo.call_args, ((100, 100), {}))


@patch('daemon.DaemonContext.open')
class DaemonizeTestCase(TestCase):
    def setUp(self):
        self.tmp_dir = mkdtemp()

    def error_maybe(self, check, fn):
        def maybe(*args, **kwargs):
            if check(*args, **kwargs):
                raise AssertionError("function called on {} {}".format(args, kwargs))
            else:
                fn(*args, **kwargs)

        return maybe

    def test_working_dir(self, dc_open):
        context = utils.daemonize("run/tests.pid", self.tmp_dir, None, None, do_open=False)
        self.assertEqual(dc_open.call_count, 0)
        self.assertEqual(context.working_directory, self.tmp_dir)
        self.assertEqual(context.chroot_directory, None)
        self.assertEqual(context.stdout.name, os.path.join(self.tmp_dir, "logs", "salmon.out"))
        self.assertEqual(context.stderr.name, os.path.join(self.tmp_dir, "logs", "salmon.err"))
        self.assertEqual(context.pidfile.path, os.path.join("run", "tests.pid"))
        self.assertEqual(context.umask, 0)

    def test_open_is_called(self, dc_open):
        context = utils.daemonize("run/tests.pid", self.tmp_dir, None, None, do_open=True)
        self.assertEqual(dc_open.call_count, 1)
        self.assertEqual(context.chroot_directory, None)
        self.assertEqual(context.stdout.name, os.path.join(self.tmp_dir, "logs", "salmon.out"))
        self.assertEqual(context.stderr.name, os.path.join(self.tmp_dir, "logs", "salmon.err"))
        self.assertEqual(context.pidfile.path, os.path.join("run", "tests.pid"))
        self.assertEqual(context.umask, 0)

    def test_chroot(self, dc_open):
        context = utils.daemonize("run/tests.pid", ".", self.tmp_dir, None, do_open=False)
        self.assertEqual(dc_open.call_count, 0)
        self.assertEqual(context.working_directory, ".")
        self.assertEqual(context.chroot_directory, self.tmp_dir)
        self.assertEqual(context.stdout.name, os.path.join(self.tmp_dir, ".", "logs", "salmon.out"))
        self.assertEqual(context.stderr.name, os.path.join(self.tmp_dir, ".", "logs", "salmon.err"))
        self.assertEqual(context.pidfile.path, os.path.join("run", "tests.pid"))
        self.assertEqual(context.umask, 0)

    def test_pid_dir_already_exists(self, dc_open):
        os.mkdir(os.path.join(self.tmp_dir, "run"))
        with patch("os.mkdir", self.error_maybe(lambda p: p.endswith("/run"), os.mkdir)):
            context = utils.daemonize("run/tests.pid", self.tmp_dir, None, None, do_open=False)
        self.assertEqual(dc_open.call_count, 0)
        self.assertEqual(context.working_directory, self.tmp_dir)
        self.assertEqual(context.stdout.name, os.path.join(self.tmp_dir, "logs", "salmon.out"))
        self.assertEqual(context.stderr.name, os.path.join(self.tmp_dir, "logs", "salmon.err"))
        self.assertEqual(context.pidfile.path, os.path.join("run", "tests.pid"))
        self.assertEqual(context.umask, 0)

    def test_logs_dir_already_exists(self, dc_open):
        os.mkdir(os.path.join(self.tmp_dir, "logs"))
        with patch("os.mkdir", self.error_maybe(lambda p: p.endswith("/logs"), os.mkdir)):
            context = utils.daemonize("run/tests.pid", self.tmp_dir, None, None, do_open=False)
        self.assertEqual(dc_open.call_count, 0)
        self.assertEqual(context.working_directory, self.tmp_dir)
        self.assertEqual(context.stdout.name, os.path.join(self.tmp_dir, "logs", "salmon.out"))
        self.assertEqual(context.stderr.name, os.path.join(self.tmp_dir, "logs", "salmon.err"))
        self.assertEqual(context.pidfile.path, os.path.join("run", "tests.pid"))
        self.assertEqual(context.umask, 0)

    def test_chdir_does_not_exist(self, dc_open):
        chdir = os.path.join(self.tmp_dir, "test")
        with self.assertRaises(OSError):
            utils.daemonize("run/tests.pid", chdir, None, None, do_open=False)
        self.assertEqual(dc_open.call_count, 0)

    def test_umask(self, dc_open):
        context = utils.daemonize("run/tests.pid", self.tmp_dir, None, 0o002, do_open=False)
        self.assertEqual(context.chroot_directory, None)
        self.assertEqual(context.stdout.name, os.path.join(self.tmp_dir, "logs", "salmon.out"))
        self.assertEqual(context.stderr.name, os.path.join(self.tmp_dir, "logs", "salmon.err"))
        self.assertEqual(context.pidfile.path, os.path.join("run", "tests.pid"))
        self.assertEqual(context.umask, 2)



from shutil import rmtree
import os
import smtplib
import subprocess
import sys
import time

from salmon import queue

from .setup_env import SalmonTestCase, dirs
from test_app.config import settings as server_settings


class IntegrationTestCase(SalmonTestCase):
    @classmethod
    def setUpClass(cls):
        cls._cwd = "tests/data/test_app"
        for path in dirs:
            rmtree(os.path.join(cls._cwd, path), ignore_errors=True)
            os.mkdir(os.path.join(cls._cwd, path))
        cls._server = subprocess.Popen(["salmon", "start", "--boot", "config.dump", "--no-daemon"],
                                       cwd=cls._cwd, stdout=sys.stdout, stderr=sys.stderr)
        for i in range(5):
            try:
                conn = smtplib.SMTP(**server_settings.receiver_config)
            except Exception:
                time.sleep(2**i)
                continue
            else:
                conn.quit()
                return

        raise Exception("Server still not ready, something must be wrong")

    @classmethod
    def tearDownClass(cls):
        cls._server.kill()
        cls._server = None
        for path in dirs:
            rmtree(os.path.join(cls._cwd, path), ignore_errors=True)

    def setUp(self):
        super(IntegrationTestCase, self).setUp()
        # re-create destoryed queues
        queue.Queue(os.path.join(self._cwd, server_settings.UNDELIVERABLE_QUEUE)).clear()
        queue.Queue(os.path.join(self._cwd, server_settings.QUEUE_PATH)).clear()

    def test_we_get_message(self):
        client = smtplib.SMTP(**server_settings.receiver_config)

        client.helo()
        client.sendmail("me@example.com", "you@example.com", "hello")

        undelivered = queue.Queue(os.path.join(self._cwd, server_settings.UNDELIVERABLE_QUEUE))
        self.assertEqual(len(undelivered), 0)

        inbox = queue.Queue(os.path.join(self._cwd, server_settings.QUEUE_PATH))
        self.assertEqual(len(inbox), 1)

    def test_we_dont_get_message(self):
        client = smtplib.SMTP(**server_settings.receiver_config)

        client.helo()
        client.sendmail("me@example.com", "you@example1.com", "hello")

        undelivered = queue.Queue(os.path.join(self._cwd, server_settings.UNDELIVERABLE_QUEUE))
        self.assertEqual(len(undelivered), 1)

        inbox = queue.Queue(os.path.join(self._cwd, server_settings.QUEUE_PATH))
        self.assertEqual(len(inbox), 0)



from mock import Mock, patch

from salmon import mail, view
from salmon.confirm import ConfirmationEngine, ConfirmationStorage
from salmon.queue import Queue
from salmon.testing import delivered, relay
import jinja2

from .setup_env import SalmonTestCase


class ConfirmationTestCase(SalmonTestCase):
    def setUp(self):
        super(ConfirmationTestCase, self).setUp()
        self.storage = ConfirmationStorage()
        self.engine = ConfirmationEngine('run/confirm', self.storage)
        view.LOADER = jinja2.Environment(loader=jinja2.FileSystemLoader('tests/data/templates'))

    def tearDown(self):
        super(ConfirmationTestCase, self).tearDown()
        view.LOADER = None

    def test_ConfirmationStorage(self):
        self.storage.store('testing', 'somedude@localhost',
                           '12345', '567890')
        secret, pending_id = self.storage.get('testing', 'somedude@localhost')
        self.assertEqual(secret, '12345')
        self.assertEqual(pending_id, '567890')

        self.storage.delete('testing', 'somedude@localhost')
        self.assertEqual(len(self.storage.confirmations), 0)

        self.storage.store('testing', 'somedude@localhost',
                           '12345', '567890')
        self.assertEqual(len(self.storage.confirmations), 1)
        self.storage.clear()
        self.assertEqual(len(self.storage.confirmations), 0)

    @patch("smtplib.SMTP")
    def test_ConfirmationEngine_send(self, smtp_mock):
        smtp_mock.return_value = Mock()

        Queue('run/queue').clear()
        self.engine.clear()

        list_name = 'testing'
        action = 'subscribing to'
        host = 'localhost'

        message = mail.MailRequest('fakepeer', 'somedude@localhost',
                                   'testing-subscribe@localhost', 'Fake body.')

        self.engine.send(relay(port=0), 'testing', message, 'confirmation.msg', locals())

        self.assertEqual(smtp_mock.return_value.sendmail.call_count, 1)
        self.assertEqual(smtp_mock.return_value.quit.call_count, 1)
        assert delivered('somedude', to_queue=self.engine.pending)

        return smtp_mock.return_value.sendmail.call_args[0][2]

    def test_ConfirmationEngine_verify(self):
        confirm = self.test_ConfirmationEngine_send()
        confirm = mail.MailRequest(None, None, None, confirm)

        resp = mail.MailRequest('fakepeer', '"Somedude Smith" <somedude@localhost>', confirm['Reply-To'], 'Fake body')

        target, _, expect_secret = confirm['Reply-To'].split('-')
        expect_secret = expect_secret.split('@')[0]

        found = self.engine.verify(target, resp['from'], 'invalid_secret')
        assert not found

        pending = self.engine.verify(target, resp['from'], expect_secret)
        assert pending, "Verify failed: %r not in %r." % (expect_secret,
                                                          self.storage.confirmations)

        self.assertEqual(pending['from'], 'somedude@localhost')
        self.assertEqual(pending['to'], 'testing-subscribe@localhost')

    def test_ConfirmationEngine_cancel(self):
        confirm = self.test_ConfirmationEngine_send()
        confirm = mail.MailRequest(None, None, None, confirm)

        target, _, expect_secret = confirm['Reply-To'].split('-')
        expect_secret = expect_secret.split('@')[0]

        self.engine.cancel(target, confirm['To'], expect_secret)

        found = self.engine.verify(target, confirm['To'], expect_secret)
        assert not found



"""
The meat of Salmon, doing all the work that actually takes an email and makes
sure that your code gets it.

The three most important parts for a programmer are the Router variable, the
StateStorage base class, and the @route, @route_like, and @stateless decorators.

The salmon.routing.Router variable (it's not a class, just named like one) is
how the whole system gets to the Router.  It is an instance of RoutingBase and
there's usually only one.

The salmon.routing.StateStorage is what you need to implement if you want
Salmon to store the state in a different way.  By default the
salmon.routing.Router object just uses a default MemoryStorage to do its job.
If you want to use a custom storage, then in your boot modiule you would set
salmon.routing.Router.STATE_STORE to what you want to use.

Finally, when you write a state handler, it has functions that act as state
functions for dealing with each state.  To tell the Router what function should
handle what email you use a @route decorator.  To tell the Route that one
function routes the same as another use @route_like.  In the case where a state
function should run on every matching email, just use the @stateless decorator
after a @route or @route_like.

If at any time you need to debug your routing setup just use the salmon routes
command.

Routing Control
===============

To control routing there are a set of decorators that you apply to your
functions.

* @route -- The main routing function that determines what addresses you are
  interested in.
* @route_like -- Says that this function routes like another one.
* @stateless -- Indicates this function always runs on each route encountered, and
  no state is maintained.
* @nolocking -- Use this if you want this handler to run parallel without any
  locking around Salmon internals.  SUPER DANGEROUS, add @stateless as well.
* @state_key_generator -- Used on a function that knows how to make your state
  keys for the module, for example if module_name + message.To is needed to maintain
  state.

It's best to put @route or @route_like as the first decorator, then the others
after that.

The @state_key_generator is different since it's not intended to go on a handler
but instead on a simple function, so it shouldn't be combined with the others.
"""
from __future__ import print_function, unicode_literals

from functools import wraps
import logging
import re
import shelve
import sys
import threading

try:
    from importlib import reload
except ImportError:
    from __builtin__ import reload


ROUTE_FIRST_STATE = 'START'
LOG = logging.getLogger("routing")


def DEFAULT_STATE_KEY(mod, msg):
    return mod


class StateStorage(object):
    """
    The base storage class you need to implement for a custom storage
    system.
    """
    def get(self, key, sender):
        """
        You must implement this so that it returns a single string
        of either the state for this combination of arguments, OR
        the ROUTE_FIRST_STATE setting.
        """
        raise NotImplementedError("You have to implement a StateStorage.get.")

    def set(self, key, sender, state):
        """
        Set should take the given parameters and consistently set the state for
        that combination such that when StateStorage.get is called it gives back
        the same setting.
        """
        raise NotImplementedError("You have to implement a StateStorage.set.")

    def clear(self):
        """
        This should clear ALL states, it is only used in unit testing, so you
        can have it raise an exception if you want to make this safer.
        """
        raise NotImplementedError("You have to implement a StateStorage.clear for unit testing to work.")


class MemoryStorage(StateStorage):
    """
    The default simplified storage for the Router to hold the states.  This
    should only be used in testing, as you'll lose all your contacts and their
    states if your server shuts down.  It is also horribly NOT thread safe.
    """
    def __init__(self):
        self.states = {}

    def get(self, key, sender):
        key = self.key(key, sender)
        try:
            return self.states[key]
        except KeyError:
            return ROUTE_FIRST_STATE

    def set(self, key, sender, state):
        key = self.key(key, sender)
        if state == ROUTE_FIRST_STATE:
            try:
                del self.states[key]
            except KeyError:
                pass
        else:
            self.states[key] = state

    def key(self, key, sender):
        return repr([key, sender])

    def clear(self):
        self.states.clear()


class ShelveStorage(MemoryStorage):
    """
    Uses Python's shelve to store the state of the Routers to disk rather than
    in memory like with MemoryStorage.  This will get you going on a small
    install if you need to persist your states (most likely), but if you
    have a database, you'll need to write your own StateStorage that
    uses your ORM or database to store.  Consider this an example.

    NOTE: Because of shelve limitations you can only use ASCII encoded keys.
    """
    def __init__(self, database_path):
        """Database path depends on the backing library use by Python's shelve."""
        self.database_path = database_path
        self.lock = threading.RLock()

    def get(self, key, sender):
        """
        This will lock the internal thread lock, and then retrieve from the
        shelf whatever key you request.  If the key is not found then it
        will set (atomically) to ROUTE_FIRST_STATE.
        """
        with self.lock:
            self.states = shelve.open(self.database_path)
            value = super(ShelveStorage, self).get(key.encode('ascii'), sender)
            self.states.close()
            return value

    def set(self, key, sender, state):
        """
        Acquires the self.lock and then sets the requested state in the shelf.
        """
        with self.lock:
            self.states = shelve.open(self.database_path)
            super(ShelveStorage, self).set(key.encode('ascii'), sender, state)
            self.states.close()

    def clear(self):
        """
        Primarily used in the debugging/unit testing process to make sure the
        states are clear.  In production this could be a bad thing.
        """
        with self.lock:
            self.states = shelve.open(self.database_path)
            super(ShelveStorage, self).clear()
            self.states.close()


class RoutingBase(object):
    """
    The self is a globally accessible class that is actually more like a
    glorified module.  It is used mostly internally by the salmon.routing
    decorators (route, route_like, stateless) to control the routing
    mechanism.

    It keeps track of the registered routes, their attached functions, the
    order that these routes should be evaluated, any default routing captures,
    and uses the MemoryStorage by default to keep track of the states.

    You can change the storage to another implementation by simple setting:

        Router.STATE_STORE = OtherStorage()

    in your settings module.

    RoutingBase does locking on every write to its internal data (which usually
    only happens during booting and reloading while debugging), and when each
    handler's state function is called.  ALL threads will go through this lock,
    but only as each state is run, so you won't have a situation where the chain
    of state functions will block all the others.  This means that while your
    handler runs nothing will be running, but you have not guarantees about
    the order of each state function.

    However, this can kill the performance of some kinds of state functions,
    so if you find the need to not have locking, then use the @nolocking
    decorator and the Router will NOT lock when that function is called.  That
    means while your @nolocking state function is running at least one other
    thread (more if the next ones happen to be @nolocking) could also be
    running.

    It's your job to keep things straight if you do that.

    NOTE: See @state_key_generator for a way to change what the key is to
    STATE_STORE for different state control options.
    """

    def __init__(self):
        self.REGISTERED = {}
        self.ORDER = []
        self.DEFAULT_CAPTURES = {}
        self.STATE_STORE = MemoryStorage()
        self.HANDLERS = {}
        self.RELOAD = False
        self.LOG_EXCEPTIONS = True
        self.UNDELIVERABLE_QUEUE = None
        self.lock = threading.RLock()
        self.call_lock = threading.RLock()

    def register_route(self, format, func):
        """
        Registers this function func into the routes mapping based on the
        format given.  Format should be a regex string ready to be handed to
        re.compile.
        """
        with self.lock:
            if format in self.REGISTERED:
                self.REGISTERED[format][1].append(func)
            else:
                self.ORDER.append(format)
                self.REGISTERED[format] = (re.compile(format, re.IGNORECASE), [func])

    def match(self, address):
        """
        This is a generator that goes through all the routes and
        yields each match it finds.  It expects you to give it a
        blah@blah.com address, NOT "Joe Blow" <blah@blah.com>.
        """
        for format in self.ORDER:
            regex, functions = self.REGISTERED[format]
            match = regex.match(address)
            if match:
                yield functions, match.groupdict()

    def defaults(self, **captures):
        """
        Updates the defaults for routing captures with the given settings.

        You use this in your handlers or your settings module to set
        common regular expressions you'll have in your @route decorators.
        This saves you typing, but also makes it easy to reconfigure later.

        For example, many times you'll have a single host="..." regex
        for all your application's routes.  Put this in your settings.py
        file using route_defaults={'host': '...'} and you're done.
        """
        with self.lock:
            self.DEFAULT_CAPTURES.update(captures)

    def get_state(self, module_name, message):
        """Returns the state that this module is in for the given message (using its from)."""
        key = self.state_key(module_name, message)
        return self.STATE_STORE.get(key, message.From)

    def in_state(self, func, message):
        """
        Determines if this function is in the state for the to/from in the
        message.  Doesn't apply to @stateless state handlers.
        """
        state = self.get_state(func.__module__, message)
        return state and state == func.__name__

    def in_error(self, func, message):
        """
        Determines if the this function is in the 'ERROR' state,
        which is a special state that self puts handlers in that throw
        an exception.
        """
        state = self.get_state(func.__module__, message)
        return state and state == 'ERROR'

    def state_key(self, module_name, message):
        """
        Given a module_name we need to get a state key for, and a
        message that has information to make the key, this function
        calls any registered @state_key_generator and returns that
        as the key.  If none is given then it just returns module_name
        as the key.
        """
        key_func = self.HANDLERS.get(module_name, DEFAULT_STATE_KEY)
        return key_func(module_name, message)

    def set_state(self, module_name, message, state):
        """
        Sets the state of the given module (a string) according to the message to the requested
        state (a string).  This is also how you can force another FSM to a required state.
        """
        key = self.state_key(module_name, message)
        self.STATE_STORE.set(key, message.From, state)

    def _collect_matches(self, message):
        in_state_found = False

        for functions, matchkw in self.match(message.To):
            for func in functions:
                if salmon_setting(func, 'stateless'):
                    yield func, matchkw
                elif not in_state_found and self.in_state(func, message):
                    in_state_found = True
                    yield func, matchkw

    def _enqueue_undeliverable(self, message):
        if self.UNDELIVERABLE_QUEUE is not None:
            LOG.debug("Message to %r from %r undeliverable, putting in undeliverable queue (# of recipients: %d).",
                      message.To, message.From, len(message.To))
            self.UNDELIVERABLE_QUEUE.push(message)
        else:
            LOG.debug("Message to %r from %r didn't match any handlers. (# recipients: %d)",
                      message.To, message.From, len(message.To))

    def deliver(self, message):
        """
        The meat of the whole Salmon operation, this method takes all the
        arguments given, and then goes through the routing listing to figure out
        which state handlers should get the gear.  The routing operates on a
        simple set of rules:

            1) Match on all functions that match the given To in their
            registered format pattern.
            2) Call all @stateless state handlers functions.
            3) Call the first method that's in the right state for the From/To.

        It will log which handlers are being run, and you can use the 'salmon route'
        command to inspect and debug routing problems.

        If you have an ERROR state function, then when your state blows up, it will
        transition to ERROR state and call your function right away.  It will then
        stay in the ERROR state unless you return a different one.
        """
        if self.RELOAD:
            self.reload()

        called_count = 0

        for func, matchkw in self._collect_matches(message):
            LOG.debug("Matched %r against %s.", message.To, func.__name__)

            if salmon_setting(func, 'nolocking'):
                self.call_safely(func, message,  matchkw)
            else:
                with self.call_lock:
                    self.call_safely(func, message, matchkw)

            called_count += 1

        if called_count == 0:
            self._enqueue_undeliverable(message)

    def call_safely(self, func, message, kwargs):
        """
        Used by self to call a function and log exceptions rather than
        explode and crash.
        """
        from salmon.server import SMTPError

        try:
            func(message, **kwargs)
            LOG.debug("Message to %s was handled by %s.%s",
                      message.To, func.__module__, func.__name__)
        except SMTPError:
            raise
        except Exception:
            self.set_state(func.__module__, message, 'ERROR')

            if self.UNDELIVERABLE_QUEUE is not None:
                self.UNDELIVERABLE_QUEUE.push(message)

            if self.LOG_EXCEPTIONS:
                LOG.exception("!!! ERROR handling %s.%s", func.__module__, func.__name__)
            else:
                raise

    def clear_states(self):
        """Clears out the states for unit testing."""
        with self.lock:
            self.STATE_STORE.clear()

    def clear_routes(self):
        """Clears out the routes for unit testing and reloading."""
        with self.lock:
            self.REGISTERED.clear()
            del self.ORDER[:]

    def load(self, handlers):
        """
        Loads the listed handlers making them available for processing.
        This is safe to call multiple times and to duplicate handlers
        listed.
        """
        with self.lock:
            for module in handlers:
                try:
                    __import__(module, globals(), locals())

                    if module not in self.HANDLERS:
                        # they didn't specify a key generator, so use the
                        # default one for now
                        self.HANDLERS[module] = DEFAULT_STATE_KEY
                except ImportError:
                    if self.LOG_EXCEPTIONS:
                        LOG.exception("ERROR IMPORTING %r MODULE:" % module)
                    else:
                        raise

    def reload(self):
        """
        Performs a reload of all the handlers and clears out all routes,
        but doesn't touch the internal state.
        """
        with self.lock:
            self.clear_routes()
            for module in list(sys.modules.keys()):
                if module in self.HANDLERS:
                    try:
                        reload(sys.modules[module])
                    except (TypeError, NameError, ImportError):
                        if self.LOG_EXCEPTIONS:
                            LOG.exception("ERROR RELOADING %r MODULE:" % module)
                        else:
                            raise


Router = RoutingBase()


class route(object):
    """
    The @route decorator is attached to state handlers to configure them in the
    Router so they handle messages for them.  The way this works is, rather than
    just routing working on only messages being sent to a state handler, it also uses
    the state of the sender.  It's like having routing in a web application use
    both the URL and an internal state setting to determine which method to run.

    However, if you'd rather than this state handler process all messages
    matching the @route then tag it @stateless.  This will run the handler
    no matter what and not change the user's state.
    """

    def __init__(self, format, **captures):
        r"""
        Sets up the pattern used for the Router configuration.  The format
        parameter is a simple pattern of words, captures, and anything you
        want to ignore.  The captures parameter is a mapping of the words in
        the format to regex that get put into the format.  When the pattern is
        matched, the captures are handed to your state handler as keyword
        arguments.

        For example, if you have::

            @route("(list_name)-(action)@(host)",
                list_name='[a-z]+',
                action='[a-z]+', host='test\.com')
            def STATE(message, list_name=None, action=None, host=None):
                pass

        Then this will be translated so that list_name is replaced with [a-z]+,
        action with [a-z]+, and host with 'test.com' to produce a regex with the
        right format and named captures to that your state handler is called
        with the proper keyword parameters.

        You should also use the Router.defaults() to set default things like the
        host so that you are not putting it into your code.
        """
        self.captures = Router.DEFAULT_CAPTURES.copy()
        self.captures.update(captures)
        self.format = self.parse_format(format, self.captures)

    def __call__(self, func):
        """Returns either a decorator that does a stateless routing or
        a normal routing."""
        self.setup_accounting(func)

        if salmon_setting(func, 'stateless'):
            @wraps(func)
            def routing_wrapper(message, *args, **kw):
                func(message, *args, **kw)
        else:
            @wraps(func)
            def routing_wrapper(message, *args, **kw):
                next_state = func(message, *args, **kw)

                if next_state:
                    Router.set_state(next_state.__module__, message, next_state.__name__)

        Router.register_route(self.format, routing_wrapper)
        return routing_wrapper

    def __get__(self, obj, of_type=None):
        """
        This is NOT SUPPORTED.  It is here just so that if you try to apply
        this decorator to a class's method it will barf on you.
        """
        raise NotImplementedError("Not supported on methods yet, only module functions.")

    def parse_format(self, format, captures):
        """Does the grunt work of conversion format+captures into the regex."""
        for key in captures:
            format = format.replace("(" + key + ")", "(?P<%s>%s)" % (key, captures[key]))
        return "^" + format + "$"

    def setup_accounting(self, func):
        """Sets up an accounting map attached to the func for routing decorators."""
        attach_salmon_settings(func)
        func._salmon_settings['format'] = self.format
        func._salmon_settings['captures'] = self.captures


def salmon_setting(func, key):
    """Simple way to get the salmon setting off the function, or None."""
    return func._salmon_settings.get(key)


def has_salmon_settings(func):
    return "_salmon_settings" in func.__dict__


def assert_salmon_settings(func):
    """Used to make sure that the func has been setup by a routing decorator."""
    assert has_salmon_settings(func), "Function %s has not be setup with a @route first." % func.__name__


def attach_salmon_settings(func):
    """Use this to setup the _salmon_settings if they aren't already there."""
    if '_salmon_settings' not in func.__dict__:
        func._salmon_settings = {}


class route_like(route):
    """
    Many times you want your state handler to just accept mail like another
    handler.  Use this, passing in the other function.  It even works across
    modules.
    """
    def __init__(self, func):
        assert_salmon_settings(func)
        self.format = func._salmon_settings['format']
        self.captures = func._salmon_settings['captures']


def stateless(func):
    """
    This simple decorator is attached to a handler to indicate to the
    Router.deliver() method that it does NOT maintain state or care about it.
    This is how you create a handler that processes all messages matching the
    given format+captures in a @route.

    Another way to think about a @stateless handler is that it is a pass-through
    handler that does its processing and then passes the results on to others.

    Stateless handlers are NOT guaranteed to run before the handler with state.
    """
    if has_salmon_settings(func):
        assert not salmon_setting(func, 'format'), "You must use @stateless AFTER @route or @route_like."

    attach_salmon_settings(func)
    func._salmon_settings['stateless'] = True

    return func


def nolocking(func):
    """
    Normally salmon.routing.Router has a lock around each call to all handlers
    to prevent them from stepping on each other.  It's assumed that 95% of the
    time this is what you want, so it's the default.  You probably want
    everything to go in order and not step on other things going off from other
    threads in the system.

    However, sometimes you know better what you are doing and this is where
    @nolocking comes in.  Put this decorator on your state functions that you
    don't care about threading issues or that you have found a need to
    manually tune, and it will run it without any locks.
    """
    attach_salmon_settings(func)
    func._salmon_settings['nolocking'] = True
    return func


def state_key_generator(func):
    """
    Used to indicate that a function in your handlers should be used
    to determine what they key is for state storage.  It should be a
    function that takes the module_name and message being worked on
    and returns a string.
    """
    Router.HANDLERS[func.__module__] = func
    return func
